@@ -12,6 +12,7 @@ if TYPE_CHECKING:
     from types import TracebackType
 
     from griptape_nodes.exe_types.node_types import BaseNode
+
 T = TypeVar("T", bound="Parameter")
 N = TypeVar("N", bound="BaseNodeElement")
 
@@ -65,25 +66,52 @@ class ParameterType:
         return ret_val
 
     @staticmethod
-    def are_types_compatible(source_type: str | None, target_type: str | None) -> bool:
+    def _extract_base_type(type_str: str) -> str:
+        """Extract the base type from a potentially generic type string.
+
+        Examples:
+            'list[any]' -> 'list'
+            'dict[str, int]' -> 'dict'
+            'str' -> 'str'
+        """
+        bracket_index = type_str.find("[")
+        if bracket_index == -1:
+            return type_str
+        return type_str[:bracket_index]
+
+    @staticmethod
+    def are_types_compatible(source_type: str | None, target_type: str | None) -> bool:  # noqa: PLR0911
         if source_type is None or target_type is None:
             return False
 
-        ret_val = False
         source_type_lower = source_type.lower()
         target_type_lower = target_type.lower()
 
         # If either are None, bail.
         if ParameterTypeBuiltin.NONE.value in (source_type_lower, target_type_lower):
-            ret_val = False
-        elif target_type_lower == ParameterTypeBuiltin.ANY.value:
+            return False
+        if target_type_lower == ParameterTypeBuiltin.ANY.value:
             # If the TARGET accepts Any, we're good. Not always true the other way 'round.
-            ret_val = True
-        else:
-            # Do a compare.
-            ret_val = source_type_lower == target_type_lower
+            return True
 
-        return ret_val
+        # First try exact match
+        if source_type_lower == target_type_lower:
+            return True
+
+        source_base = ParameterType._extract_base_type(source_type_lower)
+        target_base = ParameterType._extract_base_type(target_type_lower)
+
+        # If base types match
+        if source_base == target_base:
+            # Allow any generic to flow to base type (list[any] -> list, list[str] -> list)
+            if target_type_lower == target_base:
+                return True
+
+            # Allow specific types to flow to [any] generic (list[str] -> list[any])
+            if target_type_lower == f"{target_base}[{ParameterTypeBuiltin.ANY.value}]":
+                return True
+
+        return False
 
     @staticmethod
     def parse_kv_type_pair(type_str: str) -> KeyValueTypePair | None:  # noqa: C901
@@ -226,11 +254,12 @@ class BaseNodeElement:
 
     def _emit_alter_element_event_if_possible(self) -> None:
         """Emit an AlterElementEvent if we have node context and the necessary dependencies."""
+        from griptape_nodes.retained_mode.griptape_nodes import GriptapeNodes
+
         if self._node_context is None:
             return
 
         # Import here to avoid circular dependencies
-        from griptape.events import EventBus
 
         from griptape_nodes.retained_mode.events.base_events import ExecutionEvent, ExecutionGriptapeNodeEvent
         from griptape_nodes.retained_mode.events.parameter_events import AlterElementEvent
@@ -244,13 +273,18 @@ class BaseNodeElement:
             "name": self.name,
             "node_name": self._node_context.name,
         }
-        event_data.update(self._changes)
+        # If ui_options changed, send the complete ui_options from to_dict()
+        complete_dict = self.to_dict()
+        if "ui_options" in complete_dict:
+            self._changes["ui_options"] = complete_dict["ui_options"]
 
+        event_data.update(self._changes)
         # Publish the event
         event = ExecutionGriptapeNodeEvent(
             wrapped_event=ExecutionEvent(payload=AlterElementEvent(element_details=event_data))
         )
-        EventBus.publish_event(event)
+
+        GriptapeNodes.EventManager().put_event(event)
         self._changes.clear()
 
     def to_dict(self) -> dict[str, Any]:
@@ -295,11 +329,28 @@ class BaseNodeElement:
             self._node_context._emit_parameter_lifecycle_event(child)
 
     def remove_child(self, child: BaseNodeElement | str) -> None:
+        """Remove a child element from the hierarchy.
+
+        This method recursively searches through the element hierarchy to find and remove
+        the specified child. When the child is found in a descendant container (e.g., a
+        ParameterList), it delegates to that container's remove_child() method to ensure
+        proper cleanup and event handling (like marking parent nodes as unresolved).
+
+        Args:
+            child: The child element to remove, either as an object or by name string
+        """
         ui_elements: list[BaseNodeElement] = [self]
         for ui_element in ui_elements:
             if child in ui_element._children:
-                child._parent = None
-                ui_element._children.remove(child)
+                # Delegate to the actual parent container's remove_child method.
+                # This ensures specialized containers (like ParameterList) can perform
+                # their specific cleanup logic (e.g., marking parent nodes as unresolved).
+                if ui_element is not self:
+                    ui_element.remove_child(child)
+                else:
+                    # We are the direct parent, so handle removal directly
+                    child._parent = None
+                    ui_element._children.remove(child)
                 break
             ui_elements.extend(ui_element._children)
         if self._node_context is not None and isinstance(child, BaseNodeElement):
@@ -366,8 +417,23 @@ class BaseNodeElement:
         return event_data
 
 
-@dataclass(kw_only=True)
-class ParameterMessage(BaseNodeElement):
+class UIOptionsMixin:
+    """Mixin providing UI options update functionality for classes with ui_options."""
+
+    def update_ui_options_key(self, key: str, value: Any) -> None:
+        """Update a single UI option key."""
+        ui_options = self.ui_options
+        ui_options[key] = value
+        self.ui_options = ui_options
+
+    def update_ui_options(self, updates: dict[str, Any]) -> None:
+        """Update multiple UI options at once."""
+        ui_options = self.ui_options
+        ui_options.update(updates)
+        self.ui_options = ui_options
+
+
+class ParameterMessage(BaseNodeElement, UIOptionsMixin):
     """Represents a UI message element, such as a warning or informational text."""
 
     # Define default titles as a class-level constant
@@ -384,13 +450,97 @@ class ParameterMessage(BaseNodeElement):
     type VariantType = Literal["info", "warning", "error", "success", "tip", "none"]
 
     element_type: str = field(default_factory=lambda: ParameterMessage.__name__)
-    variant: VariantType
-    title: str | None = None
-    value: str
-    button_link: str | None = None
-    button_text: str | None = None
-    full_width: bool = False
-    ui_options: dict = field(default_factory=dict)
+    _variant: VariantType = field(init=False)
+    _title: str | None = field(default=None, init=False)
+    _value: str = field(init=False)
+    _button_link: str | None = field(default=None, init=False)
+    _button_text: str | None = field(default=None, init=False)
+    _full_width: bool = field(default=False, init=False)
+    _ui_options: dict = field(default_factory=dict, init=False)
+
+    def __init__(  # noqa: PLR0913
+        self,
+        variant: VariantType,
+        value: str,
+        *,
+        title: str | None = None,
+        button_link: str | None = None,
+        button_text: str | None = None,
+        full_width: bool = False,
+        ui_options: dict | None = None,
+        **kwargs,
+    ):
+        super().__init__(element_type=ParameterMessage.__name__, **kwargs)
+        self._variant = variant
+        self._title = title
+        self._value = value
+        self._button_link = button_link
+        self._button_text = button_text
+        self._full_width = full_width
+        self._ui_options = ui_options or {}
+
+    @property
+    def variant(self) -> VariantType:
+        return self._variant
+
+    @variant.setter
+    @BaseNodeElement.emits_update_on_write
+    def variant(self, value: VariantType) -> None:
+        self._variant = value
+
+    @property
+    def title(self) -> str | None:
+        return self._title
+
+    @title.setter
+    @BaseNodeElement.emits_update_on_write
+    def title(self, value: str | None) -> None:
+        self._title = value
+
+    @property
+    def value(self) -> str:
+        return self._value
+
+    @value.setter
+    @BaseNodeElement.emits_update_on_write
+    def value(self, value: str) -> None:
+        self._value = value
+
+    @property
+    def button_link(self) -> str | None:
+        return self._button_link
+
+    @button_link.setter
+    @BaseNodeElement.emits_update_on_write
+    def button_link(self, value: str | None) -> None:
+        self._button_link = value
+
+    @property
+    def button_text(self) -> str | None:
+        return self._button_text
+
+    @button_text.setter
+    @BaseNodeElement.emits_update_on_write
+    def button_text(self, value: str | None) -> None:
+        self._button_text = value
+
+    @property
+    def full_width(self) -> bool:
+        return self._full_width
+
+    @full_width.setter
+    @BaseNodeElement.emits_update_on_write
+    def full_width(self, value: bool) -> None:
+        self._full_width = value
+
+    @property
+    def ui_options(self) -> dict:
+        return self._ui_options
+
+    @ui_options.setter
+    @BaseNodeElement.emits_update_on_write
+    def ui_options(self, value: dict) -> None:
+        self._ui_options = value
 
     def to_dict(self) -> dict[str, Any]:
         data = super().to_dict()
@@ -429,11 +579,21 @@ class ParameterMessage(BaseNodeElement):
         return event_data
 
 
-@dataclass(kw_only=True)
-class ParameterGroup(BaseNodeElement):
+class ParameterGroup(BaseNodeElement, UIOptionsMixin):
     """UI element for a group of parameters."""
 
-    ui_options: dict = field(default_factory=dict)
+    def __init__(self, name: str, ui_options: dict | None = None, **kwargs):
+        super().__init__(name=name, **kwargs)
+        self._ui_options = ui_options or {}
+
+    @property
+    def ui_options(self) -> dict:
+        return self._ui_options
+
+    @ui_options.setter
+    @BaseNodeElement.emits_update_on_write
+    def ui_options(self, value: dict) -> None:
+        self._ui_options = value
 
     def to_dict(self) -> dict[str, Any]:
         """Returns a nested dictionary representation of this node and its children.
@@ -541,7 +701,7 @@ class ParameterBase(BaseNodeElement, ABC):
         pass
 
 
-class Parameter(BaseNodeElement):
+class Parameter(BaseNodeElement, UIOptionsMixin):
     # This is the list of types that the Parameter can accept, either externally or when internally treated as a property.
     # Today, we can accept multiple types for input, but only a single output type.
     tooltip: str | list[dict]  # Default tooltip, can be string or list of dicts
@@ -552,7 +712,15 @@ class Parameter(BaseNodeElement):
     tooltip_as_input: str | list[dict] | None = None
     tooltip_as_property: str | list[dict] | None = None
     tooltip_as_output: str | list[dict] | None = None
-    settable: bool = True
+
+    # "settable" here means whether it can be assigned to during regular business operation.
+    # During save/load, this value IS still serialized to save its proper state.
+    _settable: bool = True
+
+    # "serializable" controls whether parameter values should be serialized during save/load operations.
+    # Set to False for parameters containing non-serializable types (ImageDrivers, PromptDrivers, file handles, etc.)
+    serializable: bool = True
+
     user_defined: bool = False
     _allowed_modes: set = field(
         default_factory=lambda: {
@@ -586,6 +754,7 @@ class Parameter(BaseNodeElement):
         ui_options: dict | None = None,
         *,
         settable: bool = True,
+        serializable: bool = True,
         user_defined: bool = False,
         element_id: str | None = None,
         element_type: str | None = None,
@@ -594,7 +763,7 @@ class Parameter(BaseNodeElement):
         if not element_id:
             element_id = str(uuid.uuid4().hex)
         if not element_type:
-            element_type = BaseNodeElement.__name__
+            element_type = self.__class__.__name__
         super().__init__(element_id=element_id, element_type=element_type)
         self.name = name
         self.tooltip = tooltip
@@ -602,7 +771,8 @@ class Parameter(BaseNodeElement):
         self.tooltip_as_input = tooltip_as_input
         self.tooltip_as_property = tooltip_as_property
         self.tooltip_as_output = tooltip_as_output
-        self.settable = settable
+        self._settable = settable
+        self.serializable = serializable
         self.user_defined = user_defined
         if allowed_modes is None:
             self._allowed_modes = {ParameterMode.INPUT, ParameterMode.OUTPUT, ParameterMode.PROPERTY}
@@ -652,6 +822,8 @@ class Parameter(BaseNodeElement):
         our_dict["tooltip_as_property"] = self.tooltip_as_property
 
         our_dict["is_user_defined"] = self.user_defined
+        our_dict["settable"] = self.settable
+        our_dict["serializable"] = self.serializable
         our_dict["ui_options"] = self.ui_options
 
         # Let's bundle up the mode details.
@@ -740,6 +912,15 @@ class Parameter(BaseNodeElement):
             self._changes["mode_allowed_output"] = ParameterMode.OUTPUT in value
             self._changes["mode_allowed_property"] = ParameterMode.PROPERTY in value
 
+    @property
+    def settable(self) -> bool:
+        return self._settable
+
+    @settable.setter
+    @BaseNodeElement.emits_update_on_write
+    def settable(self, value: bool) -> None:
+        self._settable = value
+
     @property
     def ui_options(self) -> dict:
         ui_options = {}
@@ -748,7 +929,10 @@ class Parameter(BaseNodeElement):
             ui_options = ui_options | trait.ui_options_for_trait()
         ui_options = ui_options | self._ui_options
         if self._parent is not None and isinstance(self._parent, ParameterGroup):
-            ui_options = ui_options | self._parent.ui_options
+            # Access the field value directly for ParameterGroup
+            parent_ui_options = getattr(self._parent, "ui_options", {})
+            if isinstance(parent_ui_options, dict):
+                ui_options = ui_options | parent_ui_options
         return ui_options
 
     @ui_options.setter
@@ -951,6 +1135,7 @@ class ControlParameter(Parameter, ABC):
         traits: set[Trait.__class__ | Trait] | None = None,
         converters: list[Callable[[Any], Any]] | None = None,
         validators: list[Callable[[Parameter, Any], None]] | None = None,
+        ui_options: dict | None = None,
         *,
         user_defined: bool = False,
     ):
@@ -970,6 +1155,7 @@ class ControlParameter(Parameter, ABC):
             traits=traits,
             converters=converters,
             validators=validators,
+            ui_options=ui_options,
             user_defined=user_defined,
             element_type=self.__class__.__name__,
         )
@@ -980,6 +1166,7 @@ class ControlParameterInput(ControlParameter):
         self,
         tooltip: str | list[dict] = "Connection from previous node in the execution chain",
         name: str = "exec_in",
+        display_name: str | None = "Flow In",
         tooltip_as_input: str | list[dict] | None = None,
         tooltip_as_property: str | list[dict] | None = None,
         tooltip_as_output: str | list[dict] | None = None,
@@ -992,6 +1179,11 @@ class ControlParameterInput(ControlParameter):
         allowed_modes = {ParameterMode.INPUT}
         input_types = [ParameterTypeBuiltin.CONTROL_TYPE.value]
 
+        if display_name is None:
+            ui_options = None
+        else:
+            ui_options = {"display_name": display_name}
+
         # Call parent with a few explicit tweaks.
         super().__init__(
             name=name,
@@ -1005,6 +1197,7 @@ class ControlParameterInput(ControlParameter):
             traits=traits,
             converters=converters,
             validators=validators,
+            ui_options=ui_options,
             user_defined=user_defined,
         )
 
@@ -1014,6 +1207,7 @@ class ControlParameterOutput(ControlParameter):
         self,
         tooltip: str | list[dict] = "Connection to the next node in the execution chain",
         name: str = "exec_out",
+        display_name: str | None = "Flow Out",
         tooltip_as_input: str | list[dict] | None = None,
         tooltip_as_property: str | list[dict] | None = None,
         tooltip_as_output: str | list[dict] | None = None,
@@ -1026,6 +1220,11 @@ class ControlParameterOutput(ControlParameter):
         allowed_modes = {ParameterMode.OUTPUT}
         output_type = ParameterTypeBuiltin.CONTROL_TYPE.value
 
+        if display_name is None:
+            ui_options = None
+        else:
+            ui_options = {"display_name": display_name}
+
         # Call parent with a few explicit tweaks.
         super().__init__(
             name=name,
@@ -1039,6 +1238,7 @@ class ControlParameterOutput(ControlParameter):
             traits=traits,
             converters=converters,
             validators=validators,
+            ui_options=ui_options,
             user_defined=user_defined,
         )
 
@@ -1093,6 +1293,23 @@ class ParameterContainer(Parameter, ABC):
             element_type=element_type,
         )
 
+    def __bool__(self) -> bool:
+        """Parameter containers are always truthy, even when empty.
+
+        This overrides Python's default truthiness behavior for containers with __len__().
+        By default, Python makes objects with __len__() falsy when len() == 0, which
+        caused bugs where empty ParameterList/ParameterDictionary objects would fail
+        'if param' checks and fall back to stale cached values instead of computing
+        fresh empty results.
+
+        Unlike standard Python containers, ParameterContainer objects represent
+        parameter structure/definitions rather than just data, so they remain
+        meaningful even when empty.
+
+        See: https://github.com/griptape-ai/griptape-nodes/issues/1799
+        """
+        return True
+
     @abstractmethod
     def add_child_parameter(self) -> Parameter:
         pass
@@ -1155,6 +1372,27 @@ class ParameterList(ParameterContainer):
         result = f"list[{base_type}]"
         return result
 
+    def _custom_setter_for_property_type(self, value: str | None) -> None:
+        # If we are setting a type, we need to propagate this to our children as well.
+        for child in self._children:
+            if isinstance(child, Parameter):
+                child.type = value
+        super()._custom_setter_for_property_type(value)
+
+    def _custom_setter_for_property_input_types(self, value: list[str] | None) -> None:
+        # If we are setting a type, we need to propagate this to our children as well.
+        for child in self._children:
+            if isinstance(child, Parameter):
+                child.input_types = value
+        return super()._custom_setter_for_property_input_types(value)
+
+    def _custom_setter_for_property_output_type(self, value: str | None) -> None:
+        # If we are setting a type, we need to propagate this to our children as well.
+        for child in self._children:
+            if isinstance(child, Parameter):
+                child.output_type = value
+        return super()._custom_setter_for_property_output_type(value)
+
     def _custom_getter_for_property_input_types(self) -> list[str]:
         # For every valid input type, also accept a list variant of that for the CONTAINER Parameter only.
         # Children still use the input types given to them.
@@ -1224,6 +1462,112 @@ class ParameterList(ParameterContainer):
 
         return param
 
+    def clear_list(self) -> None:
+        """Remove all children that have been added to the list."""
+        children = self.find_elements_by_type(element_type=Parameter)
+        for child in children:
+            if isinstance(child, Parameter):
+                self.remove_child(child)
+                del child
+
+    # --- Convenience methods for stable list management ---
+    def get_child_parameters(self) -> list[Parameter]:
+        """Return direct child parameters only, in order of appearance."""
+        return self.find_elements_by_type(element_type=Parameter, find_recursively=False)
+
+    def append_child_parameter(self, display_name: str | None = None) -> Parameter:
+        """Append one child parameter and optionally set a display name.
+
+        This preserves existing children and adds a new one at the end.
+        """
+        child = self.add_child_parameter()
+        if display_name is not None:
+            ui_opts = child.ui_options or {}
+            ui_opts["display_name"] = display_name
+            child.ui_options = ui_opts
+        return child
+
+    def remove_last_child_parameter(self) -> None:
+        """Remove the last child parameter if one exists.
+
+        This removes from the end to preserve earlier children and their connections.
+        """
+        children = self.get_child_parameters()
+        if children:
+            last = children[-1]
+            self.remove_child(last)
+            del last
+
+    def ensure_length(self, desired_count: int, display_name_prefix: str | None = None) -> None:
+        """Grow or shrink the list to the desired length while preserving existing items.
+
+        - If increasing, appends new children to the end.
+        - If decreasing, removes children from the end.
+        - Optionally sets display names like "{prefix} 1", "{prefix} 2", ...
+        """
+        if desired_count is None:
+            return
+        try:
+            desired_count = int(desired_count)
+        except Exception:
+            desired_count = 0
+        desired_count = max(desired_count, 0)
+
+        current_children = self.get_child_parameters()
+        current_len = len(current_children)
+
+        # Grow
+        if current_len < desired_count:
+            for index in range(current_len, desired_count):
+                name = f"{display_name_prefix} {index + 1}" if display_name_prefix else None
+                self.append_child_parameter(display_name=name)
+
+        # Shrink
+        elif current_len > desired_count:
+            for _ in range(current_len - desired_count):
+                self.remove_last_child_parameter()
+
+        # Optionally re-apply display names to existing children to keep indices tidy
+        if display_name_prefix:
+            for index, child in enumerate(self.get_child_parameters()):
+                ui_opts = child.ui_options or {}
+                ui_opts["display_name"] = f"{display_name_prefix} {index + 1}"
+                child.ui_options = ui_opts
+
+    def add_child(self, child: BaseNodeElement) -> None:
+        """Override to mark parent node as unresolved when children are added.
+
+        When a ParameterList gains a child parameter, the parent node needs to be
+        marked as unresolved to trigger re-evaluation of the node's state and outputs.
+        """
+        super().add_child(child)
+
+        # Mark the parent node as unresolved since the parameter structure changed
+        if self._node_context is not None:
+            # Import at runtime to avoid circular import
+            from griptape_nodes.exe_types.node_types import NodeResolutionState
+
+            self._node_context.make_node_unresolved(
+                current_states_to_trigger_change_event={NodeResolutionState.RESOLVED, NodeResolutionState.RESOLVING}
+            )
+
+    def remove_child(self, child: BaseNodeElement | str) -> None:
+        """Override to mark parent node as unresolved when children are removed.
+
+        When a ParameterList loses a child parameter, the parent node needs to be
+        marked as unresolved to trigger re-evaluation of the node's state and outputs.
+        """
+        super().remove_child(child)
+
+        # Mark the parent node as unresolved since the parameter structure changed
+        if self._node_context is not None:
+            # Import at runtime to avoid circular import
+            from griptape_nodes.exe_types.node_types import NodeResolutionState
+
+            self._node_context.make_node_unresolved(
+                current_states_to_trigger_change_event={NodeResolutionState.RESOLVED, NodeResolutionState.RESOLVING}
+            )
+
 
 class ParameterKeyValuePair(Parameter):
     def __init__(  # noqa: PLR0913
