@@ -1,12 +1,12 @@
 from __future__ import annotations
 
+import asyncio
 import logging
 from abc import ABC, abstractmethod
 from collections.abc import Callable, Generator, Iterable
+from concurrent.futures import ThreadPoolExecutor
 from enum import StrEnum, auto
-from typing import Any, TypeVar
-
-from griptape.events import BaseEvent, EventBus
+from typing import Any, NamedTuple, TypeVar
 
 from griptape_nodes.exe_types.core_types import (
     BaseNodeElement,
@@ -26,12 +26,14 @@ from griptape_nodes.retained_mode.events.base_events import (
     ExecutionEvent,
     ExecutionGriptapeNodeEvent,
     ProgressEvent,
+    RequestPayload,
 )
 from griptape_nodes.retained_mode.events.execution_events import (
     NodeUnresolvedEvent,
     ParameterValueUpdateEvent,
 )
 from griptape_nodes.retained_mode.events.parameter_events import (
+    AddParameterToNodeRequest,
     RemoveElementEvent,
     RemoveParameterFromNodeRequest,
 )
@@ -52,6 +54,23 @@ class NodeResolutionState(StrEnum):
     RESOLVED = auto()
 
 
+class NodeMessageResult(NamedTuple):
+    """Result from a node message callback.
+
+    Attributes:
+        success: True if the message was handled successfully, False otherwise
+        details: Human-readable description of what happened
+        response: Optional response data to return to the sender
+        altered_workflow_state: True if the message handling altered workflow state.
+            Clients can use this to determine if the workflow needs to be re-saved.
+    """
+
+    success: bool
+    details: str
+    response: Any = None
+    altered_workflow_state: bool = True
+
+
 class BaseNode(ABC):
     # Owned by a flow
     name: str
@@ -68,6 +87,7 @@ class BaseNode(ABC):
     _entry_control_parameter: Parameter | None = (
         None  # The control input parameter used to enter this node during execution
     )
+    lock: bool = False  # When lock is true, the node is locked and can't be modified. When lock is false, the node is unlocked and can be modified.
 
     @property
     def parameters(self) -> list[Parameter]:
@@ -100,11 +120,14 @@ class BaseNode(ABC):
     # This is gross and we need to have a universal pass on resolution state changes and emission of events. That's what this ticket does!
     # https://github.com/griptape-ai/griptape-nodes/issues/994
     def make_node_unresolved(self, current_states_to_trigger_change_event: set[NodeResolutionState] | None) -> None:
+        from griptape_nodes.retained_mode.griptape_nodes import GriptapeNodes
+
         # See if the current state is in the set of states to trigger a change event.
         if current_states_to_trigger_change_event is not None and self.state in current_states_to_trigger_change_event:
             # Trigger the change event.
             # Send an event to the GUI so it knows this node has changed resolution state.
-            EventBus.publish_event(
+
+            GriptapeNodes.EventManager().put_event(
                 ExecutionGriptapeNodeEvent(
                     wrapped_event=ExecutionEvent(payload=NodeUnresolvedEvent(node_name=self.name))
                 )
@@ -147,6 +170,15 @@ class BaseNode(ABC):
         """Callback to confirm allowing a Connection going OUT of this Node."""
         return True
 
+    def before_incoming_connection(
+        self,
+        source_node: BaseNode,  # noqa: ARG002
+        source_parameter_name: str,  # noqa: ARG002
+        target_parameter_name: str,  # noqa: ARG002
+    ) -> None:
+        """Callback before validating a Connection coming TO this Node."""
+        return
+
     def after_incoming_connection(
         self,
         source_node: BaseNode,  # noqa: ARG002
@@ -156,6 +188,15 @@ class BaseNode(ABC):
         """Callback after a Connection has been established TO this Node."""
         return
 
+    def before_outgoing_connection(
+        self,
+        source_parameter_name: str,  # noqa: ARG002
+        target_node: BaseNode,  # noqa: ARG002
+        target_parameter_name: str,  # noqa: ARG002
+    ) -> None:
+        """Callback before validating a Connection going OUT of this Node."""
+        return
+
     def after_outgoing_connection(
         self,
         source_parameter: Parameter,  # noqa: ARG002
@@ -246,9 +287,30 @@ class BaseNode(ABC):
         # Waiting for https://github.com/griptape-ai/griptape-nodes/issues/1309
         return
 
-    def on_griptape_event(self, event: BaseEvent) -> None:  # noqa: ARG002
-        """Callback for when a Griptape Event comes destined for this Node."""
-        return
+    def on_node_message_received(
+        self,
+        optional_element_name: str | None,  # noqa: ARG002
+        message_type: str,
+        message: Any,  # noqa: ARG002
+    ) -> NodeMessageResult:
+        """Callback for when a message is sent directly to this node.
+
+        Custom nodes may elect to override this method to handle specific message types
+        and implement custom communication patterns with external systems.
+
+        Args:
+            optional_element_name: Optional element name this message relates to
+            message_type: String indicating the message type for parsing
+            message: Message payload of any type
+
+        Returns:
+            NodeMessageResult: Result containing success status, details, and optional response
+        """
+        return NodeMessageResult(
+            success=False,
+            details=f"Node '{self.name}' was sent a message of type '{message_type}'. Failed because no message handler was specified for this node. Implement the on_node_message_received method in this node class in order for it to receive messages.",
+            response=None,
+        )
 
     def does_name_exist(self, param_name: str) -> bool:
         for parameter in self.parameters:
@@ -367,9 +429,14 @@ class BaseNode(ABC):
         """
         parameter = self.get_parameter_by_name(param)
         if parameter is not None:
-            trait = parameter.find_element_by_id("Options")
-            if trait and isinstance(trait, Options):
+            # Find the Options trait by type since element_id is a UUID
+            traits = parameter.find_elements_by_type(Options)
+            if traits:
+                trait = traits[0]  # Take the first Options trait
                 trait.choices = choices
+                # Update the manually set UI options to include the new simple_dropdown
+                if hasattr(parameter, "_ui_options") and parameter._ui_options:
+                    parameter._ui_options["simple_dropdown"] = choices
 
                 if default in choices:
                     parameter.default_value = default
@@ -377,6 +444,10 @@ class BaseNode(ABC):
                 else:
                     msg = f"Default model '{default}' is not in the provided choices."
                     raise ValueError(msg)
+
+            else:
+                msg = f"No Options trait found for parameter '{param}'."
+                raise ValueError(msg)
         else:
             msg = f"Parameter '{param}' not found for updating model choices."
             raise ValueError(msg)
@@ -392,9 +463,14 @@ class BaseNode(ABC):
         """
         parameter = self.get_parameter_by_name(param)
         if parameter is not None:
-            trait = parameter.find_element_by_id("Options")
-            if trait and isinstance(trait, Options):
+            # Find the Options trait by type since element_id is a UUID
+            traits = parameter.find_elements_by_type(Options)
+            if traits:
+                trait = traits[0]  # Take the first Options trait
                 parameter.remove_trait(trait)
+            else:
+                msg = f"No Options trait found for parameter '{param}'."
+                raise ValueError(msg)
         else:
             msg = f"Parameter '{param}' not found for removing options trait."
             raise ValueError(msg)
@@ -488,7 +564,13 @@ class BaseNode(ABC):
         return None
 
     def set_parameter_value(
-        self, param_name: str, value: Any, *, initial_setup: bool = False, emit_change: bool = True
+        self,
+        param_name: str,
+        value: Any,
+        *,
+        initial_setup: bool = False,
+        emit_change: bool = True,
+        skip_before_value_set: bool = False,
     ) -> None:
         """Attempt to set a Parameter's value.
 
@@ -509,6 +591,7 @@ class BaseNode(ABC):
             value: the value intended to be set
             emit_change: whether to emit a parameter lifecycle event, defaults to True
             initial_setup: Whether this value is being set as the initial setup on the node, defaults to False. When True, the value is not given to any before/after hooks.
+            skip_before_value_set: Whether to skip the before_value_set hook, defaults to False. Used when before_value_set has already been called earlier in the flow.
 
         Returns:
             A set of parameter names within this node that were modified as a result
@@ -537,7 +620,10 @@ class BaseNode(ABC):
         # Allow custom node logic to prepare and possibly mutate the value before it is actually set.
         # Record any parameters modified for cascading.
         if not initial_setup:
-            final_value = self.before_value_set(parameter=parameter, value=candidate_value)
+            if not skip_before_value_set:
+                final_value = self.before_value_set(parameter=parameter, value=candidate_value)
+            else:
+                final_value = candidate_value
             # ACTUALLY SET THE NEW VALUE
             self.parameter_values[param_name] = final_value
 
@@ -575,7 +661,7 @@ class BaseNode(ABC):
         param = self.get_parameter_by_name(param_name)
         if param and isinstance(param, ParameterContainer):
             value = handle_container_parameter(self, param)
-            if value:
+            if value is not None:
                 return value
         if param_name in self.parameter_values:
             return self.parameter_values[param_name]
@@ -633,6 +719,41 @@ class BaseNode(ABC):
     def process[T](self) -> AsyncResult | None:
         pass
 
+    async def aprocess(self) -> None:
+        """Async version of process().
+
+        Default implementation wraps the existing process() method to maintain backwards compatibility.
+        Subclasses can override this method to provide direct async implementation.
+        """
+        result = self.process()
+
+        if result is None:
+            # Simple synchronous node - nothing to do
+            return
+
+        if isinstance(result, Generator):
+            # Handle generator pattern asynchronously using the same logic as before
+            loop = asyncio.get_running_loop()
+
+            try:
+                # Start the generator
+                func = next(result)
+
+                while True:
+                    # Run callable in thread pool (preserving existing behavior)
+                    with ThreadPoolExecutor() as executor:
+                        future_result = await loop.run_in_executor(executor, func)
+
+                    # Send result back and get next callable
+                    func = result.send(future_result)
+
+            except StopIteration:
+                # Generator is done
+                return
+        else:
+            # Some other return type - log warning but continue
+            logger.warning("Node %s process() returned unexpected type: %s", self.name, type(result))
+
     # if not implemented, it will return no issues.
     def validate_before_workflow_run(self) -> list[Exception] | None:
         """Runs before the entire workflow is run."""
@@ -691,6 +812,8 @@ class BaseNode(ABC):
         self.current_spotlight_parameter = None
 
     def append_value_to_parameter(self, parameter_name: str, value: Any) -> None:
+        from griptape_nodes.retained_mode.griptape_nodes import GriptapeNodes
+
         # Add the value to the node
         if parameter_name in self.parameter_output_values:
             try:
@@ -704,9 +827,14 @@ class BaseNode(ABC):
         else:
             self.parameter_output_values[parameter_name] = value
         # Publish the event up!
-        EventBus.publish_event(ProgressEvent(value=value, node_name=self.name, parameter_name=parameter_name))
+
+        GriptapeNodes.EventManager().put_event(
+            ProgressEvent(value=value, node_name=self.name, parameter_name=parameter_name)
+        )
 
     def publish_update_to_parameter(self, parameter_name: str, value: Any) -> None:
+        from griptape_nodes.retained_mode.griptape_nodes import GriptapeNodes
+
         parameter = self.get_parameter_by_name(parameter_name)
         if parameter:
             data_type = parameter.type
@@ -717,7 +845,10 @@ class BaseNode(ABC):
                 data_type=data_type,
                 value=TypeValidator.safe_serialize(value),
             )
-            EventBus.publish_event(ExecutionGriptapeNodeEvent(wrapped_event=ExecutionEvent(payload=payload)))
+
+            GriptapeNodes.EventManager().put_event(
+                ExecutionGriptapeNodeEvent(wrapped_event=ExecutionEvent(payload=payload))
+            )
         else:
             msg = f"Parameter '{parameter_name} doesn't exist on {self.name}'"
             raise RuntimeError(msg)
@@ -830,20 +961,22 @@ class BaseNode(ABC):
         """Emit an AlterElementEvent for parameter add/remove operations."""
         from griptape_nodes.retained_mode.events.base_events import ExecutionEvent, ExecutionGriptapeNodeEvent
         from griptape_nodes.retained_mode.events.parameter_events import AlterElementEvent
+        from griptape_nodes.retained_mode.griptape_nodes import GriptapeNodes
 
         # Create event data using the parameter's to_event method
         if remove:
+            # Import logger here to avoid circular dependency
             event = ExecutionGriptapeNodeEvent(
                 wrapped_event=ExecutionEvent(payload=RemoveElementEvent(element_id=parameter.element_id))
             )
         else:
             event_data = parameter.to_event(self)
-
             # Publish the event
             event = ExecutionGriptapeNodeEvent(
                 wrapped_event=ExecutionEvent(payload=AlterElementEvent(element_details=event_data))
             )
-        EventBus.publish_event(event)
+
+        GriptapeNodes.EventManager().put_event(event)
 
     def _get_element_name(self, element: str | int, element_names: list[str]) -> str:
         """Convert an element identifier (name or index) to its name.
@@ -984,6 +1117,7 @@ class TrackedParameterOutputValues(dict[str, Any]):
         if parameter is not None:
             from griptape_nodes.retained_mode.events.base_events import ExecutionEvent, ExecutionGriptapeNodeEvent
             from griptape_nodes.retained_mode.events.parameter_events import AlterElementEvent
+            from griptape_nodes.retained_mode.griptape_nodes import GriptapeNodes
 
             # Create event data using the parameter's to_event method
             event_data = parameter.to_event(self._node)
@@ -996,7 +1130,8 @@ class TrackedParameterOutputValues(dict[str, Any]):
             event = ExecutionGriptapeNodeEvent(
                 wrapped_event=ExecutionEvent(payload=AlterElementEvent(element_details=event_data))
             )
-            EventBus.publish_event(event)
+
+            GriptapeNodes.EventManager().put_event(event)
 
 
 class ControlNode(BaseNode):
@@ -1038,17 +1173,235 @@ class EndNode(BaseNode):
 
 
 class StartLoopNode(BaseNode):
-    finished: bool
-    current_index: int
     end_node: EndLoopNode | None = None
     """Creating class for Start Loop Node in order to implement loop functionality in execution."""
 
+    @abstractmethod
+    def is_loop_finished(self) -> bool:
+        """Return True if the loop has finished executing.
+
+        This method must be implemented by subclasses to define when
+        the loop should terminate.
+        """
+
 
 class EndLoopNode(BaseNode):
     start_node: StartLoopNode | None = None
     """Creating class for Start Loop Node in order to implement loop functionality in execution."""
 
 
+class ErrorProxyNode(BaseNode):
+    """A proxy node that substitutes for nodes that failed to create due to missing dependencies or errors.
+
+    This node maintains the original node type information and allows workflows to continue loading
+    even when some node types are unavailable. It generates parameters dynamically as connections
+    and values are assigned to maintain workflow structure.
+    """
+
+    def __init__(
+        self,
+        name: str,
+        original_node_type: str,
+        original_library_name: str,
+        failure_reason: str,
+        metadata: dict[Any, Any] | None = None,
+    ) -> None:
+        super().__init__(name, metadata)
+
+        self.original_node_type = original_node_type
+        self.original_library_name = original_library_name
+        self.failure_reason = failure_reason
+        # Record ALL initial_setup=True requests in order for 1:1 replay
+        self._recorded_initialization_requests: list[RequestPayload] = []
+
+        # Track if user has made connection modifications after initial setup
+        self._has_connection_modifications: bool = False
+
+        # Add error message parameter explaining the failure
+        self._error_message = ParameterMessage(
+            name="error_proxy_message",
+            variant="error",
+            value="",  # Will be set by _update_error_message
+        )
+        self.add_node_element(self._error_message)
+        self._update_error_message()
+
+    def _get_base_error_message(self) -> str:
+        """Generate the base error message for this ErrorProxyNode."""
+        return (
+            f"This is a placeholder for a node of type '{self.original_node_type}'"
+            f"\nfrom the '{self.original_library_name}' library."
+            f"\nIt encountered a problem when loading."
+            f"\nThe technical issue:\n{self.failure_reason}\n\n"
+            f"Your original node will be restored once the issue above is fixed "
+            f"(which may require registering the appropriate library, or getting "
+            f"a code fix from the node author)."
+        )
+
+    def on_attempt_set_parameter_value(self, param_name: str) -> None:
+        """Public method to attempt setting a parameter value during initial setup.
+
+        Creates a PROPERTY mode parameter if it doesn't exist to support value setting.
+
+        Args:
+            param_name: Name of the parameter to prepare for value setting
+        """
+        self._ensure_parameter_exists(param_name)
+
+    def _ensure_parameter_exists(self, param_name: str) -> None:
+        """Ensures a parameter exists on this node.
+
+        Creates a universal parameter with all modes enabled for maximum flexibility.
+        Auto-generated parameters are marked as non-user-defined so they don't get serialized.
+
+        Args:
+            param_name: Name of the parameter to ensure exists
+        """
+        existing_param = super().get_parameter_by_name(param_name)
+
+        if existing_param is None:
+            # Create new universal parameter with all modes enabled
+            from griptape_nodes.retained_mode.griptape_nodes import GriptapeNodes
+
+            request = AddParameterToNodeRequest(
+                node_name=self.name,
+                parameter_name=param_name,
+                type=ParameterTypeBuiltin.ANY.value,  # ANY = parameter's main type for maximum flexibility
+                input_types=[ParameterTypeBuiltin.ANY.value],  # ANY = accepts any single input type
+                output_type=ParameterTypeBuiltin.ALL.value,  # ALL = can output any type (passthrough)
+                tooltip="Parameter created for placeholder node to preserve workflow connections",
+                mode_allowed_input=True,  # Enable all modes upfront
+                mode_allowed_output=True,
+                mode_allowed_property=True,
+                is_user_defined=False,  # Don't serialize this parameter
+                initial_setup=True,  # Allows setting non-settable parameters and prevents resolution cascades during workflow loading
+            )
+            result = GriptapeNodes.handle_request(request)
+
+            # Check if parameter creation was successful
+            from griptape_nodes.retained_mode.events.parameter_events import AddParameterToNodeResultSuccess
+
+            if not isinstance(result, AddParameterToNodeResultSuccess):
+                failure_message = f"Failed to create parameter '{param_name}': {result.result_details}"
+                raise RuntimeError(failure_message)
+        # If parameter already exists, nothing to do - it already has all modes
+
+    def allow_incoming_connection(
+        self,
+        source_node: BaseNode,  # noqa: ARG002
+        source_parameter: Parameter,  # noqa: ARG002
+        target_parameter: Parameter,  # noqa: ARG002
+    ) -> bool:
+        """ErrorProxyNode allows connections - it's a shell for maintaining connections."""
+        return True
+
+    def allow_outgoing_connection(
+        self,
+        source_parameter: Parameter,  # noqa: ARG002
+        target_node: BaseNode,  # noqa: ARG002
+        target_parameter: Parameter,  # noqa: ARG002
+    ) -> bool:
+        """ErrorProxyNode allows connections - it's a shell for maintaining connections."""
+        return True
+
+    def before_incoming_connection(
+        self,
+        source_node: BaseNode,  # noqa: ARG002
+        source_parameter_name: str,  # noqa: ARG002
+        target_parameter_name: str,
+    ) -> None:
+        """Create target parameter before connection validation."""
+        self._ensure_parameter_exists(target_parameter_name)
+
+    def before_outgoing_connection(
+        self,
+        source_parameter_name: str,
+        target_node: BaseNode,  # noqa: ARG002
+        target_parameter_name: str,  # noqa: ARG002
+    ) -> None:
+        """Create source parameter before connection validation."""
+        self._ensure_parameter_exists(source_parameter_name)
+
+    def set_post_init_connections_modified(self) -> None:
+        """Mark that user-initiated connections have been modified and update the warning message."""
+        if not self._has_connection_modifications:
+            self._has_connection_modifications = True
+            self._update_error_message()
+
+    def _update_error_message(self) -> None:
+        """Update the ParameterMessage to include connection modification warning."""
+        # Build the updated message with connection warning
+        base_message = self._get_base_error_message()
+
+        # Add connection modification warning if applicable
+        if self._has_connection_modifications:
+            connection_warning = (
+                "\n\nWARNING: You have modified connections to this placeholder node."
+                "\nThis may require manual fixes when the original node is restored."
+            )
+            final_message = base_message + connection_warning
+        else:
+            # Add the general note only if no modifications have been made
+            general_warning = (
+                "\n\nNote: Making changes to this node may require manual fixes when restored,"
+                "\nas we can't predict how all node authors craft their custom nodes."
+            )
+            final_message = base_message + general_warning
+
+        # Update the error message value
+        self._error_message.value = final_message
+
+    def validate_before_node_run(self) -> list[Exception] | None:
+        """Prevent ErrorProxy nodes from running - validate at node level only."""
+        error_msg = (
+            f"Cannot run node '{self.name}': This is a placeholder node put in place to preserve your workflow until the breaking issue is fixed.\n\n"
+            f"The original '{self.original_node_type}' from library '{self.original_library_name}' failed to load due to this technical issue:\n\n"
+            f"{self.failure_reason}\n\n"
+            f"Once you resolve the issue above, reload this workflow and the placeholder will be automatically replaced with the original node."
+        )
+        return [RuntimeError(error_msg)]
+
+    def record_initialization_request(self, request: RequestPayload) -> None:
+        """Record an initialization request for replay during serialization.
+
+        This method captures requests that modify ErrorProxyNode structure during workflow loading,
+        preserving information needed for restoration when the original node becomes available.
+
+        WHAT WE RECORD:
+        - AlterParameterDetailsRequest: Parameter modifications from original node definition
+        - Any request with initial_setup=True that changes node structure in ways that cannot
+          be reconstructed from final state alone
+
+        WHAT WE DO NOT RECORD (and why):
+        - SetParameterValueRequest: Final parameter values are serialized normally via parameter_values
+        - AddParameterToNodeRequest: User-defined parameters are serialized via is_user_defined=True flag
+        - CreateConnectionRequest: Connections are serialized separately and recreated during loading
+        - RenameParameterRequest: Final parameter names are preserved in serialized state
+        - SetNodeMetadataRequest: Final metadata state is preserved in node.metadata
+        - SetLockNodeStateRequest: Final lock state is preserved in node.lock
+        """
+        self._recorded_initialization_requests.append(request)
+
+    def get_recorded_initialization_requests(self, request_type: type | None = None) -> list[RequestPayload]:
+        """Get recorded initialization requests for 1:1 serialization replay.
+
+        Args:
+            request_type: Optional class to filter by. If provided, only returns requests
+                         of that type. If None, returns all recorded requests.
+
+        Returns:
+            List of recorded requests in the order they were received.
+        """
+        if request_type is None:
+            return self._recorded_initialization_requests
+
+        return [req for req in self._recorded_initialization_requests if isinstance(req, request_type)]
+
+    def process(self) -> Any:
+        """No-op process method. Error Proxy nodes do nothing during execution."""
+        return None
+
+
 class Connection:
     source_node: BaseNode
     target_node: BaseNode
