@@ -1,6 +1,5 @@
 from __future__ import annotations
 
-import importlib.metadata
 import logging
 import os
 import re
@@ -37,17 +36,21 @@ from griptape_nodes.retained_mode.events.app_events import (
     SetEngineNameResultSuccess,
 )
 from griptape_nodes.retained_mode.events.base_events import (
-    AppPayload,
-    RequestPayload,
-    ResultPayload,
+    GriptapeNodeEvent,
     ResultPayloadFailure,
 )
 from griptape_nodes.retained_mode.events.flow_events import (
     DeleteFlowRequest,
 )
 from griptape_nodes.utils.metaclasses import SingletonMeta
+from griptape_nodes.utils.version_utils import engine_version
 
 if TYPE_CHECKING:
+    from griptape_nodes.retained_mode.events.base_events import (
+        AppPayload,
+        RequestPayload,
+        ResultPayload,
+    )
     from griptape_nodes.retained_mode.managers.agent_manager import AgentManager
     from griptape_nodes.retained_mode.managers.arbitrary_code_exec_manager import (
         ArbitraryCodeExecManager,
@@ -69,6 +72,10 @@ if TYPE_CHECKING:
     from griptape_nodes.retained_mode.managers.static_files_manager import (
         StaticFilesManager,
     )
+    from griptape_nodes.retained_mode.managers.sync_manager import SyncManager
+    from griptape_nodes.retained_mode.managers.variable_manager import (
+        VariablesManager,
+    )
     from griptape_nodes.retained_mode.managers.version_compatibility_manager import (
         VersionCompatibilityManager,
     )
@@ -78,9 +85,6 @@ if TYPE_CHECKING:
 logger = logging.getLogger("griptape_nodes")
 
 
-engine_version = importlib.metadata.version("griptape_nodes")
-
-
 @dataclass
 class Version:
     major: int
@@ -134,6 +138,7 @@ class GriptapeNodes(metaclass=SingletonMeta):
     _context_manager: ContextManager
     _library_manager: LibraryManager
     _workflow_manager: WorkflowManager
+    _workflow_variables_manager: VariablesManager
     _arbitrary_code_exec_manager: ArbitraryCodeExecManager
     _operation_depth_manager: OperationDepthManager
     _static_files_manager: StaticFilesManager
@@ -141,6 +146,7 @@ class GriptapeNodes(metaclass=SingletonMeta):
     _version_compatibility_manager: VersionCompatibilityManager
     _session_manager: SessionManager
     _engine_identity_manager: EngineIdentityManager
+    _sync_manager: SyncManager
 
     def __init__(self) -> None:
         from griptape_nodes.retained_mode.managers.agent_manager import AgentManager
@@ -164,6 +170,10 @@ class GriptapeNodes(metaclass=SingletonMeta):
         from griptape_nodes.retained_mode.managers.static_files_manager import (
             StaticFilesManager,
         )
+        from griptape_nodes.retained_mode.managers.sync_manager import SyncManager
+        from griptape_nodes.retained_mode.managers.variable_manager import (
+            VariablesManager,
+        )
         from griptape_nodes.retained_mode.managers.version_compatibility_manager import (
             VersionCompatibilityManager,
         )
@@ -183,6 +193,7 @@ class GriptapeNodes(metaclass=SingletonMeta):
             self._context_manager = ContextManager(self._event_manager)
             self._library_manager = LibraryManager(self._event_manager)
             self._workflow_manager = WorkflowManager(self._event_manager)
+            self._workflow_variables_manager = VariablesManager(self._event_manager)
             self._arbitrary_code_exec_manager = ArbitraryCodeExecManager(self._event_manager)
             self._operation_depth_manager = OperationDepthManager(self._config_manager)
             self._static_files_manager = StaticFilesManager(
@@ -192,6 +203,7 @@ class GriptapeNodes(metaclass=SingletonMeta):
             self._version_compatibility_manager = VersionCompatibilityManager(self._event_manager)
             self._session_manager = SessionManager(self._event_manager)
             self._engine_identity_manager = EngineIdentityManager(self._event_manager)
+            self._sync_manager = SyncManager(self._event_manager, self._config_manager)
 
             # Assign handlers now that these are created.
             self._event_manager.assign_manager_to_request_type(
@@ -225,33 +237,53 @@ class GriptapeNodes(metaclass=SingletonMeta):
     def handle_request(
         cls,
         request: RequestPayload,
-        *,
-        response_topic: str | None = None,
-        request_id: str | None = None,
     ) -> ResultPayload:
+        """Synchronous request handler."""
         event_mgr = GriptapeNodes.EventManager()
-        obj_depth_mgr = GriptapeNodes.OperationDepthManager()
-        workflow_mgr = GriptapeNodes.WorkflowManager()
+
         try:
-            return event_mgr.handle_request(
-                request=request,
-                operation_depth_mgr=obj_depth_mgr,
-                workflow_mgr=workflow_mgr,
-                response_topic=response_topic,
-                request_id=request_id,
-            )
+            result_event = event_mgr.handle_request(request=request)
+            event_mgr.put_event(GriptapeNodeEvent(wrapped_event=result_event))
         except Exception as e:
             logger.exception(
                 "Unhandled exception while processing request of type %s. "
-                "Consider saving your work and restarting the engine if issues persist.",
+                "Consider saving your work and restarting the engine if issues persist."
+                "Request: %s",
                 type(request).__name__,
+                request,
             )
             return ResultPayloadFailure(exception=e)
+        else:
+            return result_event.result
 
     @classmethod
-    def broadcast_app_event(cls, app_event: AppPayload) -> None:
+    async def ahandle_request(cls, request: RequestPayload) -> ResultPayload:
+        """Asynchronous request handler.
+
+        Args:
+            request: The request payload to handle.
+        """
+        event_mgr = GriptapeNodes.EventManager()
+
+        try:
+            result_event = await event_mgr.ahandle_request(request=request)
+            await event_mgr.aput_event(GriptapeNodeEvent(wrapped_event=result_event))
+        except Exception as e:
+            logger.exception(
+                "Unhandled exception while processing async request of type %s. "
+                "Consider saving your work and restarting the engine if issues persist."
+                "Request: %s",
+                type(request).__name__,
+                request,
+            )
+            return ResultPayloadFailure(exception=e)
+        else:
+            return result_event.result
+
+    @classmethod
+    async def broadcast_app_event(cls, app_event: AppPayload) -> None:
         event_mgr = GriptapeNodes.get_instance()._event_manager
-        return event_mgr.broadcast_app_event(app_event)
+        await event_mgr.broadcast_app_event(app_event)
 
     @classmethod
     def get_session_id(cls) -> str | None:
@@ -329,6 +361,14 @@ class GriptapeNodes(metaclass=SingletonMeta):
     def EngineIdentityManager(cls) -> EngineIdentityManager:
         return GriptapeNodes.get_instance()._engine_identity_manager
 
+    @classmethod
+    def SyncManager(cls) -> SyncManager:
+        return GriptapeNodes.get_instance()._sync_manager
+
+    @classmethod
+    def VariablesManager(cls) -> VariablesManager:
+        return GriptapeNodes.get_instance()._workflow_variables_manager
+
     @classmethod
     def clear_data(cls) -> None:
         # Get canvas
@@ -352,16 +392,16 @@ class GriptapeNodes(metaclass=SingletonMeta):
             msg = "Failed to successfully delete all objects"
             raise ValueError(msg)
 
-    def on_app_connection_established(self, _payload: AppConnectionEstablished) -> None:
+    async def on_app_connection_established(self, _payload: AppConnectionEstablished) -> None:
         from griptape_nodes.app.app import subscribe_to_topic
 
         # Subscribe to request topic (engine discovery)
-        subscribe_to_topic("request")
+        await subscribe_to_topic("request")
 
         # Get engine ID and subscribe to engine_id/request
         engine_id = GriptapeNodes.get_engine_id()
         if engine_id:
-            subscribe_to_topic(f"engines/{engine_id}/request")
+            await subscribe_to_topic(f"engines/{engine_id}/request")
         else:
             logger.warning("Engine ID not available for subscription")
 
@@ -369,7 +409,7 @@ class GriptapeNodes(metaclass=SingletonMeta):
         session_id = GriptapeNodes.get_session_id()
         if session_id:
             topic = f"sessions/{session_id}/request"
-            subscribe_to_topic(topic)
+            await subscribe_to_topic(topic)
         else:
             logger.info("No session ID available for subscription")
 
@@ -384,13 +424,13 @@ class GriptapeNodes(metaclass=SingletonMeta):
                 )
             details = f"Attempted to get engine version. Failed because version string '{engine_ver}' wasn't in expected major.minor.patch format."
             logger.error(details)
-            return GetEngineVersionResultFailure()
+            return GetEngineVersionResultFailure(result_details=details)
         except Exception as err:
             details = f"Attempted to get engine version. Failed due to '{err}'."
             logger.error(details)
-            return GetEngineVersionResultFailure()
+            return GetEngineVersionResultFailure(result_details=details)
 
-    def handle_session_start_request(self, request: AppStartSessionRequest) -> ResultPayload:  # noqa: ARG002
+    async def handle_session_start_request(self, request: AppStartSessionRequest) -> ResultPayload:  # noqa: ARG002
         from griptape_nodes.app.app import subscribe_to_topic
 
         current_session_id = GriptapeNodes.SessionManager().get_active_session_id()
@@ -404,12 +444,12 @@ class GriptapeNodes(metaclass=SingletonMeta):
             details = f"Session '{current_session_id}' already active. Joining..."
 
         topic = f"sessions/{current_session_id}/request"
-        subscribe_to_topic(topic)
+        await subscribe_to_topic(topic)
         logger.info("Subscribed to new session topic: %s", topic)
 
         return AppStartSessionResultSuccess(current_session_id)
 
-    def handle_session_end_request(self, _: AppEndSessionRequest) -> ResultPayload:
+    async def handle_session_end_request(self, _: AppEndSessionRequest) -> ResultPayload:
         from griptape_nodes.app.app import unsubscribe_from_topic
 
         try:
@@ -423,13 +463,13 @@ class GriptapeNodes(metaclass=SingletonMeta):
                 GriptapeNodes.SessionManager().clear_saved_session()
 
             unsubscribe_topic = f"sessions/{previous_session_id}/request"
-            unsubscribe_from_topic(unsubscribe_topic)
+            await unsubscribe_from_topic(unsubscribe_topic)
 
             return AppEndSessionResultSuccess(session_id=previous_session_id)
         except Exception as err:
             details = f"Failed to end session due to '{err}'."
             logger.error(details)
-            return AppEndSessionResultFailure()
+            return AppEndSessionResultFailure(result_details=details)
 
     def handle_get_session_request(self, _: AppGetSessionRequest) -> ResultPayload:
         return AppGetSessionResultSuccess(session_id=GriptapeNodes.SessionManager().get_active_session_id())
@@ -442,14 +482,17 @@ class GriptapeNodes(metaclass=SingletonMeta):
         try:
             active_session_id = GriptapeNodes.SessionManager().get_active_session_id()
             if active_session_id is None:
-                logger.warning("Session heartbeat received but no active session found")
-                return SessionHeartbeatResultFailure()
+                details = "Session heartbeat received but no active session found"
+                logger.warning(details)
+                return SessionHeartbeatResultFailure(result_details=details)
 
-            logger.debug("Session heartbeat successful for session: %s", active_session_id)
+            details = f"Session heartbeat successful for session: {active_session_id}"
+            logger.debug(details)
             return SessionHeartbeatResultSuccess()
         except Exception as err:
-            logger.error("Failed to handle session heartbeat: %s", err)
-            return SessionHeartbeatResultFailure()
+            details = f"Failed to handle session heartbeat: {err}"
+            logger.error(details)
+            return SessionHeartbeatResultFailure(result_details=details)
 
     def handle_engine_heartbeat_request(self, request: EngineHeartbeatRequest) -> ResultPayload:
         """Handle engine heartbeat requests.
@@ -478,8 +521,9 @@ class GriptapeNodes(metaclass=SingletonMeta):
                 **workflow_info,
             )
         except Exception as err:
-            logger.error("Failed to handle engine heartbeat: %s", err)
-            return EngineHeartbeatResultFailure(heartbeat_id=request.heartbeat_id)
+            details = f"Failed to handle engine heartbeat: {err}"
+            logger.error(details)
+            return EngineHeartbeatResultFailure(heartbeat_id=request.heartbeat_id, result_details=details)
 
     def handle_get_engine_name_request(self, request: GetEngineNameRequest) -> ResultPayload:  # noqa: ARG002
         """Handle requests to get the current engine name."""
@@ -490,7 +534,7 @@ class GriptapeNodes(metaclass=SingletonMeta):
         except Exception as err:
             error_message = f"Failed to get engine name: {err}"
             logger.error(error_message)
-            return GetEngineNameResultFailure(error_message=error_message)
+            return GetEngineNameResultFailure(error_message=error_message, result_details=error_message)
 
     def handle_set_engine_name_request(self, request: SetEngineNameRequest) -> ResultPayload:
         """Handle requests to set a new engine name."""
@@ -499,7 +543,7 @@ class GriptapeNodes(metaclass=SingletonMeta):
             if not request.engine_name or not request.engine_name.strip():
                 error_message = "Engine name cannot be empty"
                 logger.warning(error_message)
-                return SetEngineNameResultFailure(error_message=error_message)
+                return SetEngineNameResultFailure(error_message=error_message, result_details=error_message)
 
             # Set the new engine name
             GriptapeNodes.EngineIdentityManager().set_engine_name(request.engine_name.strip())
@@ -509,7 +553,7 @@ class GriptapeNodes(metaclass=SingletonMeta):
         except Exception as err:
             error_message = f"Failed to set engine name: {err}"
             logger.error(error_message)
-            return SetEngineNameResultFailure(error_message=error_message)
+            return SetEngineNameResultFailure(error_message=error_message, result_details=error_message)
 
     def _get_instance_info(self) -> dict[str, str | None]:
         """Get instance information from environment variables.
