@@ -9,7 +9,9 @@ from pydantic import ValidationError
 from xdg_base_dirs import xdg_config_home
 
 from griptape_nodes.retained_mode.events.app_events import AppInitializationComplete
-from griptape_nodes.retained_mode.events.base_events import ResultPayload
+from griptape_nodes.retained_mode.events.base_events import (
+    ResultPayload,
+)
 from griptape_nodes.retained_mode.events.config_events import (
     GetConfigCategoryRequest,
     GetConfigCategoryResultFailure,
@@ -254,7 +256,7 @@ class ConfigManager:
         existing_workflows = self.get_config_value(config_loc)
         if not existing_workflows:
             existing_workflows = []
-        existing_workflows.append(workflow_file_name)
+        existing_workflows.append(workflow_file_name) if workflow_file_name not in existing_workflows else None
         self.set_config_value(config_loc, existing_workflows)
 
     def delete_user_workflow(self, workflow_file_name: str) -> None:
@@ -353,39 +355,33 @@ class ConfigManager:
         if request.category is None or request.category == "":
             # Return the whole shebang. Start with the defaults and then layer on the user config.
             contents = self.merged_config
-            details = "Successfully returned the entire config dictionary."
-            logger.debug(details)
-            return GetConfigCategoryResultSuccess(contents=contents)
+            result_details = "Successfully returned the entire config dictionary."
+            return GetConfigCategoryResultSuccess(contents=contents, result_details=result_details)
 
         # See if we got something valid.
         find_results = self.get_config_value(request.category)
         if find_results is None:
-            details = f"Attempted to get config details for category '{request.category}'. Failed because no such category could be found."
-            logger.error(details)
-            return GetConfigCategoryResultFailure()
+            result_details = f"Attempted to get config details for category '{request.category}'. Failed because no such category could be found."
+            return GetConfigCategoryResultFailure(result_details=result_details)
 
         if not isinstance(find_results, dict):
-            details = f"Attempted to get config details for category '{request.category}'. Failed because this was was not a dictionary."
-            logger.error(details)
-            return GetConfigCategoryResultFailure()
+            result_details = f"Attempted to get config details for category '{request.category}'. Failed because this was was not a dictionary."
+            return GetConfigCategoryResultFailure(result_details=result_details)
 
-        details = f"Successfully returned the config dictionary for section '{request.category}'."
-        logger.debug(details)
-        return GetConfigCategoryResultSuccess(contents=find_results)
+        result_details = f"Successfully returned the config dictionary for section '{request.category}'."
+        return GetConfigCategoryResultSuccess(contents=find_results, result_details=result_details)
 
     def on_handle_set_config_category_request(self, request: SetConfigCategoryRequest) -> ResultPayload:
         # Validate the value is a dict
         if not isinstance(request.contents, dict):
-            details = f"Attempted to set config details for category '{request.category}'. Failed because the contents provided were not a dictionary."
-            logger.error(details)
-            return SetConfigCategoryResultFailure()
+            result_details = f"Attempted to set config details for category '{request.category}'. Failed because the contents provided were not a dictionary."
+            return SetConfigCategoryResultFailure(result_details=result_details)
 
         if request.category is None or request.category == "":
             # Assign the whole shebang.
             self._write_user_config_delta(request.contents)
-            details = "Successfully assigned the entire config dictionary."
-            logger.debug(details)
-            return SetConfigCategoryResultSuccess()
+            result_details = "Successfully assigned the entire config dictionary."
+            return SetConfigCategoryResultSuccess(result_details=result_details)
 
         self.set_config_value(key=request.category, value=request.contents)
 
@@ -395,29 +391,26 @@ class ConfigManager:
         for after_env_var in after_env_vars_set:
             self._update_secret_from_env_var(after_env_var)
 
-        details = f"Successfully assigned the config dictionary for section '{request.category}'."
-        logger.debug(details)
-        return SetConfigCategoryResultSuccess()
+        result_details = f"Successfully assigned the config dictionary for section '{request.category}'."
+        return SetConfigCategoryResultSuccess(result_details=result_details)
 
     def on_handle_get_config_value_request(self, request: GetConfigValueRequest) -> ResultPayload:
         if request.category_and_key == "":
-            details = "Attempted to get config value but no category or key was specified."
-            logger.error(details)
-            return GetConfigValueResultFailure()
+            result_details = "Attempted to get config value but no category or key was specified."
+            return GetConfigValueResultFailure(result_details=result_details)
 
         # See if we got something valid.
         find_results = self.get_config_value(request.category_and_key)
         if find_results is None:
-            details = f"Attempted to get config value for category.key '{request.category_and_key}'. Failed because no such category.key could be found."
-            logger.error(details)
-            return GetConfigValueResultFailure()
+            result_details = f"Attempted to get config value for category.key '{request.category_and_key}'. Failed because no such category.key could be found."
+            return GetConfigValueResultFailure(result_details=result_details)
 
-        details = f"Successfully returned the config value for section '{request.category_and_key}'."
-        logger.debug(details)
-        return GetConfigValueResultSuccess(value=find_results)
+        result_details = f"Successfully returned the config value for section '{request.category_and_key}'."
+        return GetConfigValueResultSuccess(value=find_results, result_details=result_details)
 
     def on_handle_get_config_path_request(self, request: GetConfigPathRequest) -> ResultPayload:  # noqa: ARG002
-        return GetConfigPathResultSuccess(config_path=str(USER_CONFIG_PATH))
+        result_details = "Successfully returned the config path."
+        return GetConfigPathResultSuccess(config_path=str(USER_CONFIG_PATH), result_details=result_details)
 
     def on_handle_reset_config_request(self, request: ResetConfigRequest) -> ResultPayload:  # noqa: ARG002
         try:
@@ -425,11 +418,11 @@ class ConfigManager:
             self._set_log_level(str(self.merged_config["log_level"]))
             self.workspace_path = Path(self.merged_config["workspace_directory"])
 
-            return ResetConfigResultSuccess()
+            result_details = "Successfully reset user configuration."
+            return ResetConfigResultSuccess(result_details=result_details)
         except Exception as e:
-            details = f"Attempted to reset user configuration but failed: {e}."
-            logger.error(details)
-            return ResetConfigResultFailure()
+            result_details = f"Attempted to reset user configuration but failed: {e}."
+            return ResetConfigResultFailure(result_details=result_details)
 
     def _get_diff(self, old_value: Any, new_value: Any) -> dict[Any, Any]:
         """Generate a diff between the old and new values."""
@@ -469,9 +462,8 @@ class ConfigManager:
 
     def on_handle_set_config_value_request(self, request: SetConfigValueRequest) -> ResultPayload:
         if request.category_and_key == "":
-            details = "Attempted to set config value but no category or key was specified."
-            logger.error(details)
-            return SetConfigValueResultFailure()
+            result_details = "Attempted to set config value but no category or key was specified."
+            return SetConfigValueResultFailure(result_details=result_details)
 
         # Fetch the existing value (don't go to the env vars directly; we want the key)
         old_value = self.get_config_value(request.category_and_key, should_load_env_var_if_detected=False)
@@ -497,16 +489,15 @@ class ConfigManager:
                 diff = self._get_diff(old_value_copy, request.value)
                 formatted_diff = self._format_diff(diff)
                 if formatted_diff:
-                    details = f"Successfully updated {type(request.value).__name__} at '{request.category_and_key}'. Changes:\n{formatted_diff}"
+                    result_details = f"Successfully updated {type(request.value).__name__} at '{request.category_and_key}'. Changes:\n{formatted_diff}"
                 else:
-                    details = f"Successfully updated {type(request.value).__name__} at '{request.category_and_key}'. No changes detected."
+                    result_details = f"Successfully updated {type(request.value).__name__} at '{request.category_and_key}'. No changes detected."
             else:
-                details = f"Successfully updated {type(request.value).__name__} at '{request.category_and_key}'"
+                result_details = f"Successfully updated {type(request.value).__name__} at '{request.category_and_key}'"
         else:
-            details = f"Successfully assigned the config value for '{request.category_and_key}':\n\tFROM '{old_value_copy}'\n\tTO: '{request.value}'"
+            result_details = f"Successfully assigned the config value for '{request.category_and_key}':\n\tFROM '{old_value_copy}'\n\tTO: '{request.value}'"
 
-        logger.debug(details)
-        return SetConfigValueResultSuccess()
+        return SetConfigValueResultSuccess(result_details=result_details)
 
     def _write_user_config_delta(self, user_config_delta: dict) -> None:
         """Write the user configuration to the config file.
