@@ -13,7 +13,6 @@ from importlib.resources import files
 from pathlib import Path
 from typing import TYPE_CHECKING, cast
 
-import uv
 from packaging.requirements import InvalidRequirement, Requirement
 from pydantic import ValidationError
 from rich.align import Align
@@ -96,6 +95,9 @@ from griptape_nodes.retained_mode.managers.library_lifecycle.library_provenance.
 )
 from griptape_nodes.retained_mode.managers.library_lifecycle.library_status import LibraryStatus
 from griptape_nodes.retained_mode.managers.os_manager import OSManager
+from griptape_nodes.utils.async_utils import subprocess_run
+from griptape_nodes.utils.uv_utils import find_uv_bin
+from griptape_nodes.utils.version_utils import get_complete_version_string
 
 if TYPE_CHECKING:
     from collections.abc import Callable
@@ -109,21 +111,6 @@ logger = logging.getLogger("griptape_nodes")
 console = Console()
 
 
-def _find_griptape_uv_bin() -> str:
-    """Find the uv binary, checking dedicated Griptape installation first, then system uv.
-
-    Returns:
-        Path to the uv binary to use
-    """
-    # Check for dedicated Griptape uv installation first
-    dedicated_uv_path = xdg_data_home() / "griptape_nodes" / "bin" / "uv"
-    if dedicated_uv_path.exists():
-        return str(dedicated_uv_path)
-
-    # Fall back to system uv installation
-    return uv.find_uv_bin()
-
-
 class LibraryManager:
     SANDBOX_LIBRARY_NAME = "Sandbox Library"
 
@@ -254,10 +241,10 @@ class LibraryManager:
 
         # Status emojis mapping
         status_emoji = {
-            LibraryStatus.GOOD: "✅",
-            LibraryStatus.FLAWED: "🟡",
-            LibraryStatus.UNUSABLE: "❌",
-            LibraryStatus.MISSING: "❓",
+            LibraryStatus.GOOD: "[green]OK[/green]",
+            LibraryStatus.FLAWED: "[yellow]![/yellow]",
+            LibraryStatus.UNUSABLE: "[red]X[/red]",
+            LibraryStatus.MISSING: "[red]?[/red]",
         }
 
         # Add rows for each library info
@@ -270,7 +257,7 @@ class LibraryManager:
             # Library name column with emoji based on status
             emoji = status_emoji.get(lib_info.status, "ERROR: Unknown/Unexpected Library Status")
             name = lib_info.library_name if lib_info.library_name else "*UNKNOWN*"
-            library_name = f"{emoji} {name}"
+            library_name = f"{emoji} - {name}"
 
             library_version = lib_info.library_version
             if library_version:
@@ -329,9 +316,8 @@ class LibraryManager:
         """Get all registered event handlers for a specific request type."""
         request_type = PayloadRegistry.get_type(request.request_type)
         if request_type is None:
-            return ListCapableLibraryEventHandlersResultFailure(
-                exception=KeyError(f"Request type '{request.request_type}' is not registered in the PayloadRegistry.")
-            )
+            details = f"Request type '{request.request_type}' is not registered in the PayloadRegistry."
+            return ListCapableLibraryEventHandlersResultFailure(exception=KeyError(details), result_details=details)
         handler_mappings = self.get_registered_event_handlers(request_type)
         return ListCapableLibraryEventHandlersResultSuccess(handlers=list(handler_mappings.keys()))
 
@@ -356,7 +342,7 @@ class LibraryManager:
             details = f"Attempted to list node types in a Library named '{request.library}'. Failed because no Library with that name was registered."
             logger.error(details)
 
-            result = ListNodeTypesInLibraryResultFailure()
+            result = ListNodeTypesInLibraryResultFailure(result_details=details)
             return result
 
         # Cool, get a copy of the list.
@@ -379,7 +365,7 @@ class LibraryManager:
             details = f"Attempted to get metadata for Library '{request.library}'. Failed because no Library with that name was registered."
             logger.error(details)
 
-            result = GetLibraryMetadataResultFailure()
+            result = GetLibraryMetadataResultFailure(result_details=details)
             return result
 
         # Get the metadata off of it.
@@ -412,6 +398,7 @@ class LibraryManager:
                 problems=[
                     "Library could not be found at the file path specified. It will be removed from the configuration."
                 ],
+                result_details=details,
             )
 
         # Load the JSON
@@ -426,6 +413,7 @@ class LibraryManager:
                 library_name=None,
                 status=LibraryStatus.UNUSABLE,
                 problems=["Library file not formatted as proper JSON."],
+                result_details=details,
             )
         except Exception as err:
             details = f"Attempted to load Library JSON file from location '{json_path}'. Failed because an exception occurred: {err}"
@@ -435,6 +423,7 @@ class LibraryManager:
                 library_name=None,
                 status=LibraryStatus.UNUSABLE,
                 problems=[f"Exception occurred when attempting to load the library: {err}."],
+                result_details=details,
             )
 
         # Try to extract library name from JSON for better error reporting
@@ -459,6 +448,7 @@ class LibraryManager:
                 library_name=library_name,
                 status=LibraryStatus.UNUSABLE,
                 problems=problems,
+                result_details=details,
             )
         except Exception as err:
             details = f"Attempted to load Library JSON file. Failed because the file at path '{json_path}' failed to match the library schema due to: {err}"
@@ -468,6 +458,7 @@ class LibraryManager:
                 library_name=library_name,
                 status=LibraryStatus.UNUSABLE,
                 problems=[f"Library file did not match the library schema specified due to: {err}"],
+                result_details=details,
             )
 
         details = f"Successfully loaded library metadata from JSON file at {json_path}"
@@ -537,11 +528,13 @@ class LibraryManager:
         sandbox_library_dir_as_posix = sandbox_library_dir.as_posix()
 
         if not sandbox_library_dir.exists():
+            details = "Sandbox directory does not exist."
             return LoadLibraryMetadataFromFileResultFailure(
                 library_path=sandbox_library_dir_as_posix,
                 library_name=LibraryManager.SANDBOX_LIBRARY_NAME,
                 status=LibraryStatus.MISSING,
-                problems=["Sandbox directory does not exist."],
+                problems=[details],
+                result_details=details,
             )
 
         sandbox_node_candidates = self._find_files_in_dir(directory=sandbox_library_dir, extension=".py")
@@ -592,11 +585,13 @@ class LibraryManager:
 
         engine_version = GriptapeNodes().handle_engine_version_request(request=GetEngineVersionRequest())
         if not isinstance(engine_version, GetEngineVersionResultSuccess):
+            details = "Could not get engine version for sandbox library generation."
             return LoadLibraryMetadataFromFileResultFailure(
                 library_path=sandbox_library_dir_as_posix,
                 library_name=LibraryManager.SANDBOX_LIBRARY_NAME,
                 status=LibraryStatus.UNUSABLE,
-                problems=["Could not get engine version for sandbox library generation."],
+                problems=[details],
+                result_details=details,
             )
 
         engine_version_str = f"{engine_version.major}.{engine_version.minor}.{engine_version.patch}"
@@ -633,7 +628,7 @@ class LibraryManager:
             details = f"Attempted to get node metadata for a node type '{request.node_type}' in a Library named '{request.library}'. Failed because no Library with that name was registered."
             logger.error(details)
 
-            result = GetNodeMetadataFromLibraryResultFailure()
+            result = GetNodeMetadataFromLibraryResultFailure(result_details=details)
             return result
 
         # Does the node type exist within the library?
@@ -643,7 +638,7 @@ class LibraryManager:
             details = f"Attempted to get node metadata for a node type '{request.node_type}' in a Library named '{request.library}'. Failed because no node type of that name could be found in the Library."
             logger.error(details)
 
-            result = GetNodeMetadataFromLibraryResultFailure()
+            result = GetNodeMetadataFromLibraryResultFailure(result_details=details)
             return result
 
         details = f"Successfully retrieved node metadata for a node type '{request.node_type}' in a Library named '{request.library}'."
@@ -661,14 +656,14 @@ class LibraryManager:
         except KeyError:
             details = f"Attempted to get categories in a Library named '{request.library}'. Failed because no Library with that name was registered."
             logger.error(details)
-            result = ListCategoriesInLibraryResultFailure()
+            result = ListCategoriesInLibraryResultFailure(result_details=details)
             return result
 
         categories = library.get_categories()
         result = ListCategoriesInLibraryResultSuccess(categories=categories)
         return result
 
-    def register_library_from_file_request(self, request: RegisterLibraryFromFileRequest) -> ResultPayload:  # noqa: C901, PLR0911, PLR0912, PLR0915 (complex logic needs branches)
+    async def register_library_from_file_request(self, request: RegisterLibraryFromFileRequest) -> ResultPayload:  # noqa: C901, PLR0911, PLR0912, PLR0915 (complex logic needs branches)
         file_path = request.file_path
 
         # Convert to Path object if it's a string
@@ -686,7 +681,7 @@ class LibraryManager:
             )
             details = f"Attempted to load Library JSON file. Failed because no file could be found at the specified path: {json_path}"
             logger.error(details)
-            return RegisterLibraryFromFileResultFailure()
+            return RegisterLibraryFromFileResultFailure(result_details=details)
 
         # Use the new metadata loading functionality
         metadata_request = LoadLibraryMetadataFromFileRequest(file_path=file_path)
@@ -702,7 +697,7 @@ class LibraryManager:
                 status=failure_result.status,
                 problems=failure_result.problems,
             )
-            return RegisterLibraryFromFileResultFailure()
+            return RegisterLibraryFromFileResultFailure(result_details=str(failure_result.result_details))
 
         # Get the validated library data
         library_data = metadata_result.library_schema
@@ -720,7 +715,7 @@ class LibraryManager:
             )
             details = f"Attempted to load Library '{library_data.name}' JSON file from '{json_path}'. Failed because version string '{library_data.metadata.library_version}' wasn't valid. Must be in major.minor.patch format."
             logger.error(details)
-            return RegisterLibraryFromFileResultFailure()
+            return RegisterLibraryFromFileResultFailure(result_details=details)
 
         # Get the directory containing the JSON file to resolve relative paths
         base_dir = json_path.parent.absolute()
@@ -747,7 +742,7 @@ class LibraryManager:
                 )
                 details = f"Attempted to load Library '{library_data.name}' from '{json_path}'. Failed to load Advanced Library module: {err}"
                 logger.error(details)
-                return RegisterLibraryFromFileResultFailure()
+                return RegisterLibraryFromFileResultFailure(result_details=details)
 
         # Create or get the library
         try:
@@ -772,7 +767,7 @@ class LibraryManager:
 
             details = f"Attempted to load Library JSON file from '{json_path}'. Failed because a Library '{library_data.name}' already exists. Error: {err}."
             logger.error(details)
-            return RegisterLibraryFromFileResultFailure()
+            return RegisterLibraryFromFileResultFailure(result_details=details)
 
         # Install node library dependencies
         try:
@@ -787,7 +782,7 @@ class LibraryManager:
 
                 # Only install dependencies if conditions are met
                 try:
-                    library_venv_python_path = self._init_library_venv(venv_path)
+                    library_venv_python_path = await self._init_library_venv(venv_path)
                 except RuntimeError as e:
                     self._library_file_path_to_info[file_path] = LibraryManager.LibraryInfo(
                         library_path=file_path,
@@ -798,19 +793,15 @@ class LibraryManager:
                     )
                     details = f"Attempted to load Library JSON file from '{json_path}'. Failed when creating the virtual environment: {e}."
                     logger.error(details)
-                    return RegisterLibraryFromFileResultFailure()
+                    return RegisterLibraryFromFileResultFailure(result_details=details)
                 if self._can_write_to_venv_location(library_venv_python_path):
                     # Check disk space before installing dependencies
                     config_manager = GriptapeNodes.ConfigManager()
                     min_space_gb = config_manager.get_config_value("minimum_disk_space_gb_libraries")
                     if not OSManager.check_available_disk_space(Path(venv_path), min_space_gb):
                         error_msg = OSManager.format_disk_space_error(Path(venv_path))
-                        logger.error(
-                            "Attempted to load Library JSON from '%s'. Failed when installing dependencies (requires %.1f GB): %s",
-                            json_path,
-                            min_space_gb,
-                            error_msg,
-                        )
+                        details = f"Attempted to load Library JSON from '{json_path}'. Failed when installing dependencies due to insufficient disk space (requires {min_space_gb} GB): {error_msg}"
+                        logger.error(details)
                         self._library_file_path_to_info[file_path] = LibraryManager.LibraryInfo(
                             library_path=file_path,
                             library_name=library_data.name,
@@ -820,13 +811,13 @@ class LibraryManager:
                                 f"Insufficient disk space for dependencies (requires {min_space_gb} GB): {error_msg}"
                             ],
                         )
-                        return RegisterLibraryFromFileResultFailure()
+                        return RegisterLibraryFromFileResultFailure(result_details=details)
 
                     # Grab the python executable from the virtual environment so that we can pip install there
                     logger.info(
                         "Installing dependencies for library '%s' with pip in venv at %s", library_data.name, venv_path
                     )
-                    subprocess.run(  # noqa: S603
+                    await subprocess_run(
                         [
                             sys.executable,
                             "-m",
@@ -861,7 +852,7 @@ class LibraryManager:
             )
             details = f"Attempted to load Library JSON file from '{json_path}'. Failed when installing dependencies: {error_details}"
             logger.error(details)
-            return RegisterLibraryFromFileResultFailure()
+            return RegisterLibraryFromFileResultFailure(result_details=details)
 
         # We are at least potentially viable.
         # Record all problems that occurred
@@ -924,13 +915,13 @@ class LibraryManager:
             case LibraryStatus.UNUSABLE:
                 details = f"Attempted to load Library JSON file from '{json_path}'. Failed because no nodes were loaded. Check the log for more details."
                 logger.error(details)
-                return RegisterLibraryFromFileResultFailure()
+                return RegisterLibraryFromFileResultFailure(result_details=details)
             case _:
                 details = f"Attempted to load Library JSON file from '{json_path}'. Failed because an unknown/unexpected status '{library_load_results.status}' was returned."
                 logger.error(details)
-                return RegisterLibraryFromFileResultFailure()
+                return RegisterLibraryFromFileResultFailure(result_details=details)
 
-    def register_library_from_requirement_specifier_request(
+    async def register_library_from_requirement_specifier_request(
         self, request: RegisterLibraryFromRequirementSpecifierRequest
     ) -> ResultPayload:
         try:
@@ -940,34 +931,32 @@ class LibraryManager:
 
             # Only install dependencies if conditions are met
             try:
-                library_python_venv_path = self._init_library_venv(venv_path)
+                library_python_venv_path = await self._init_library_venv(venv_path)
             except RuntimeError as e:
                 details = f"Attempted to install library '{request.requirement_specifier}'. Failed when creating the virtual environment: {e}"
                 logger.error(details)
-                return RegisterLibraryFromRequirementSpecifierResultFailure()
+                return RegisterLibraryFromRequirementSpecifierResultFailure(result_details=details)
             if self._can_write_to_venv_location(library_python_venv_path):
                 # Check disk space before installing dependencies
                 config_manager = GriptapeNodes.ConfigManager()
                 min_space_gb = config_manager.get_config_value("minimum_disk_space_gb_libraries")
                 if not OSManager.check_available_disk_space(Path(venv_path), min_space_gb):
                     error_msg = OSManager.format_disk_space_error(Path(venv_path))
-                    logger.error(
-                        "Attempted to install library '%s'. Failed when installing dependencies (requires %.1f GB): %s",
-                        request.requirement_specifier,
-                        min_space_gb,
-                        error_msg,
-                    )
-                    return RegisterLibraryFromRequirementSpecifierResultFailure()
+                    details = f"Attempted to install library '{request.requirement_specifier}'. Failed when installing dependencies due to insufficient disk space (requires {min_space_gb} GB): {error_msg}"
+                    logger.error(details)
+                    return RegisterLibraryFromRequirementSpecifierResultFailure(result_details=details)
+
+                uv_path = find_uv_bin()
 
                 logger.info("Installing dependency '%s' with pip in venv at %s", package_name, venv_path)
-                subprocess.run(  # noqa: S603
+                await subprocess_run(
                     [
-                        _find_griptape_uv_bin(),
+                        uv_path,
                         "pip",
                         "install",
                         request.requirement_specifier,
                         "--python",
-                        library_python_venv_path,
+                        str(library_python_venv_path),
                     ],
                     check=True,
                     capture_output=True,
@@ -982,11 +971,11 @@ class LibraryManager:
         except subprocess.CalledProcessError as e:
             details = f"Attempted to install library '{request.requirement_specifier}'. Failed: return code={e.returncode}, stdout={e.stdout}, stderr={e.stderr}"
             logger.error(details)
-            return RegisterLibraryFromRequirementSpecifierResultFailure()
+            return RegisterLibraryFromRequirementSpecifierResultFailure(result_details=details)
         except InvalidRequirement as e:
             details = f"Attempted to install library '{request.requirement_specifier}'. Failed due to invalid requirement specifier: {e}"
             logger.error(details)
-            return RegisterLibraryFromRequirementSpecifierResultFailure()
+            return RegisterLibraryFromRequirementSpecifierResultFailure(result_details=details)
 
         library_path = str(files(package_name).joinpath(request.library_config_name))
 
@@ -994,11 +983,11 @@ class LibraryManager:
         if isinstance(register_result, RegisterLibraryFromFileResultFailure):
             details = f"Attempted to install library '{request.requirement_specifier}'. Failed due to {register_result}"
             logger.error(details)
-            return RegisterLibraryFromRequirementSpecifierResultFailure()
+            return RegisterLibraryFromRequirementSpecifierResultFailure(result_details=details)
 
         return RegisterLibraryFromRequirementSpecifierResultSuccess(library_name=request.requirement_specifier)
 
-    def _init_library_venv(self, library_venv_path: Path) -> Path:
+    async def _init_library_venv(self, library_venv_path: Path) -> Path:
         """Initialize a virtual environment for the library.
 
         If the virtual environment already exists, it will not be recreated.
@@ -1032,9 +1021,10 @@ class LibraryManager:
                 raise RuntimeError(error_message)
 
             try:
+                uv_path = find_uv_bin()
                 logger.info("Creating virtual environment at %s with Python %s", library_venv_path, python_version)
-                subprocess.run(  # noqa: S603
-                    [sys.executable, "-m", "uv", "venv", str(library_venv_path), "--python", python_version],
+                await subprocess_run(
+                    [uv_path, "venv", str(library_venv_path), "--python", python_version],
                     check=True,
                     capture_output=True,
                     text=True,
@@ -1120,7 +1110,7 @@ class LibraryManager:
         except Exception as e:
             details = f"Attempted to unload library '{request.library_name}'. Failed due to {e}"
             logger.error(details)
-            return UnloadLibraryFromRegistryResultFailure()
+            return UnloadLibraryFromRegistryResultFailure(result_details=details)
 
         # Clean up all stable module aliases for this library
         self._unregister_all_stable_module_aliases_for_library(request.library_name)
@@ -1142,7 +1132,7 @@ class LibraryManager:
         if not list_libraries_result.succeeded():
             details = "Attempted to get all info for all libraries, but listing the registered libraries failed."
             logger.error(details)
-            return GetAllInfoForAllLibrariesResultFailure()
+            return GetAllInfoForAllLibrariesResultFailure(result_details=details)
 
         try:
             list_libraries_success = cast("ListRegisteredLibrariesResultSuccess", list_libraries_result)
@@ -1157,7 +1147,7 @@ class LibraryManager:
                 if not library_all_info_result.succeeded():
                     details = f"Attempted to get all info for all libraries, but failed when getting all info for library named '{library_name}'."
                     logger.error(details)
-                    return GetAllInfoForAllLibrariesResultFailure()
+                    return GetAllInfoForAllLibrariesResultFailure(result_details=details)
 
                 library_all_info_success = cast("GetAllInfoForLibraryResultSuccess", library_all_info_result)
 
@@ -1165,7 +1155,7 @@ class LibraryManager:
         except Exception as err:
             details = f"Attempted to get all info for all libraries. Encountered error {err}."
             logger.error(details)
-            return GetAllInfoForAllLibrariesResultFailure()
+            return GetAllInfoForAllLibrariesResultFailure(result_details=details)
 
         # We're home free now
         details = "Successfully retrieved all info for all libraries."
@@ -1180,7 +1170,7 @@ class LibraryManager:
         except KeyError:
             details = f"Attempted to get all library info for a Library named '{request.library}'. Failed because no Library with that name was registered."
             logger.error(details)
-            result = GetAllInfoForLibraryResultFailure()
+            result = GetAllInfoForLibraryResultFailure(result_details=details)
             return result
 
         library_metadata_request = GetLibraryMetadataRequest(library=request.library)
@@ -1189,7 +1179,7 @@ class LibraryManager:
         if not library_metadata_result.succeeded():
             details = f"Attempted to get all library info for a Library named '{request.library}'. Failed attempting to get the library's metadata."
             logger.error(details)
-            return GetAllInfoForLibraryResultFailure()
+            return GetAllInfoForLibraryResultFailure(result_details=details)
 
         list_categories_request = ListCategoriesInLibraryRequest(library=request.library)
         list_categories_result = self.list_categories_in_library_request(list_categories_request)
@@ -1197,7 +1187,7 @@ class LibraryManager:
         if not list_categories_result.succeeded():
             details = f"Attempted to get all library info for a Library named '{request.library}'. Failed attempting to get the list of categories in the library."
             logger.error(details)
-            return GetAllInfoForLibraryResultFailure()
+            return GetAllInfoForLibraryResultFailure(result_details=details)
 
         node_type_list_request = ListNodeTypesInLibraryRequest(library=request.library)
         node_type_list_result = self.on_list_node_types_in_library_request(node_type_list_request)
@@ -1205,7 +1195,7 @@ class LibraryManager:
         if not node_type_list_result.succeeded():
             details = f"Attempted to get all library info for a Library named '{request.library}'. Failed attempting to get the list of node types in the library."
             logger.error(details)
-            return GetAllInfoForLibraryResultFailure()
+            return GetAllInfoForLibraryResultFailure(result_details=details)
 
         # Cast everyone to their success counterparts.
         try:
@@ -1217,7 +1207,7 @@ class LibraryManager:
                 f"Attempted to get all library info for a Library named '{request.library}'. Encountered error: {err}."
             )
             logger.error(details)
-            return GetAllInfoForLibraryResultFailure()
+            return GetAllInfoForLibraryResultFailure(result_details=details)
 
         # Now build the map of node types to metadata.
         node_type_name_to_node_metadata_details = {}
@@ -1228,14 +1218,14 @@ class LibraryManager:
             if not node_metadata_result.succeeded():
                 details = f"Attempted to get all library info for a Library named '{request.library}'. Failed attempting to get the metadata for a node type called '{node_type_name}'."
                 logger.error(details)
-                return GetAllInfoForLibraryResultFailure()
+                return GetAllInfoForLibraryResultFailure(result_details=details)
 
             try:
                 node_metadata_result_success = cast("GetNodeMetadataFromLibraryResultSuccess", node_metadata_result)
             except Exception as err:
                 details = f"Attempted to get all library info for a Library named '{request.library}'. Encountered error: {err}."
                 logger.error(details)
-                return GetAllInfoForLibraryResultFailure()
+                return GetAllInfoForLibraryResultFailure(result_details=details)
 
             # Put it into the map.
             node_type_name_to_node_metadata_details[node_type_name] = node_metadata_result_success
@@ -1502,7 +1492,7 @@ class LibraryManager:
 
         return node_class
 
-    def load_all_libraries_from_config(self) -> None:
+    async def load_all_libraries_from_config(self) -> None:
         # Comment out lines 1503-1545 and call the _load libraries from provenance system to test the other functionality.
 
         # Load metadata for all libraries to determine which ones can be safely loaded
@@ -1538,7 +1528,7 @@ class LibraryManager:
                 register_request = RegisterLibraryFromFileRequest(
                     file_path=library_result.file_path, load_as_default_library=False
                 )
-                register_result = self.register_library_from_file_request(register_request)
+                register_result = await self.register_library_from_file_request(register_request)
                 if isinstance(register_result, RegisterLibraryFromFileResultFailure):
                     # Registration failed - the failure info is already recorded in _library_file_path_to_info
                     # by register_library_from_file_request, so we just log it here for visibility
@@ -1551,12 +1541,12 @@ class LibraryManager:
         user_libraries_section = "app_events.on_app_initialization_complete.libraries_to_register"
         self._remove_missing_libraries_from_config(config_category=user_libraries_section)
 
-    def on_app_initialization_complete(self, _payload: AppInitializationComplete) -> None:
+    async def on_app_initialization_complete(self, _payload: AppInitializationComplete) -> None:
         GriptapeNodes.EngineIdentityManager().initialize_engine_id()
         GriptapeNodes.SessionManager().get_saved_session_id()
 
         # App just got init'd. See if there are library JSONs to load!
-        self.load_all_libraries_from_config()
+        await self.load_all_libraries_from_config()
 
         # We have to load all libraries before we attempt to load workflows.
 
@@ -1594,14 +1584,7 @@ class LibraryManager:
         GriptapeNodes.WorkflowManager().on_libraries_initialization_complete()
 
         # Print the engine ready message
-        engine_version_request = GetEngineVersionRequest()
-        engine_version_result = GriptapeNodes.get_instance().handle_engine_version_request(engine_version_request)
-        if isinstance(engine_version_result, GetEngineVersionResultSuccess):
-            engine_version = (
-                f"v{engine_version_result.major}.{engine_version_result.minor}.{engine_version_result.patch}"
-            )
-        else:
-            engine_version = "<UNKNOWN ENGINE VERSION>"
+        engine_version = get_complete_version_string()
 
         # Get current session ID
         session_id = GriptapeNodes.get_session_id()
@@ -1614,14 +1597,14 @@ class LibraryManager:
                 f"[bold blue]Return to: [link={nodes_app_url}]{nodes_app_url}[/link] to access the Workflow Editor[/bold blue]",
                 vertical="middle",
             ),
-            title="🚀 Griptape Nodes Engine Started",
+            title="Griptape Nodes Engine Started",
             subtitle=f"[green]{engine_version}{session_info}[/green]",
             border_style="green",
             padding=(1, 4),
         )
         console.print(message)
 
-    def _load_libraries_from_provenance_system(self) -> None:
+    async def _load_libraries_from_provenance_system(self) -> None:
         """Load libraries using the new provenance-based system with FSM.
 
         This method converts libraries_to_register entries into LibraryProvenanceLocalFile
@@ -1650,7 +1633,7 @@ class LibraryManager:
 
             # Add to directory as user candidate (defaults to active=True)
             # This automatically creates FSM and runs evaluation
-            self._library_directory.add_user_candidate(provenance)
+            await self._library_directory.add_user_candidate(provenance)
 
             logger.debug("Added library provenance: %s", provenance.get_display_name())
 
@@ -1679,8 +1662,8 @@ class LibraryManager:
 
         # Process installable candidates through installation and loading
         for candidate in installable_candidates:
-            if self._library_directory.install_library(candidate.provenance):
-                self._library_directory.load_library(candidate.provenance)
+            if await self._library_directory.install_library(candidate.provenance):
+                await self._library_directory.load_library(candidate.provenance)
 
     def _report_library_name_conflicts(self) -> None:
         """Report on library name conflicts found during evaluation."""
@@ -2015,14 +1998,14 @@ class LibraryManager:
             ]
             config_mgr.set_config_value(config_category, libraries_to_register_category)
 
-    def reload_all_libraries_request(self, request: ReloadAllLibrariesRequest) -> ResultPayload:  # noqa: ARG002
+    async def reload_all_libraries_request(self, request: ReloadAllLibrariesRequest) -> ResultPayload:  # noqa: ARG002
         # Start with a clean slate.
         clear_all_request = ClearAllObjectStateRequest(i_know_what_im_doing=True)
         clear_all_result = GriptapeNodes.handle_request(clear_all_request)
         if not clear_all_result.succeeded():
             details = "Failed to clear the existing object state when preparing to reload all libraries."
             logger.error(details)
-            return ReloadAllLibrariesResultFailure()
+            return ReloadAllLibrariesResultFailure(result_details=details)
 
         # Unload all libraries now.
         all_libraries_request = ListRegisteredLibrariesRequest()
@@ -2030,7 +2013,7 @@ class LibraryManager:
         if not isinstance(all_libraries_result, ListRegisteredLibrariesResultSuccess):
             details = "When preparing to reload all libraries, failed to get registered libraries."
             logger.error(details)
-            return ReloadAllLibrariesResultFailure()
+            return ReloadAllLibrariesResultFailure(result_details=details)
 
         for library_name in all_libraries_result.libraries:
             unload_library_request = UnloadLibraryFromRegistryRequest(library_name=library_name)
@@ -2038,10 +2021,10 @@ class LibraryManager:
             if not unload_library_result.succeeded():
                 details = f"When preparing to reload all libraries, failed to unload library '{library_name}'."
                 logger.error(details)
-                return ReloadAllLibrariesResultFailure()
+                return ReloadAllLibrariesResultFailure(result_details=details)
 
         # Load (or reload, which should trigger a hot reload) all libraries
-        self.load_all_libraries_from_config()
+        await self.load_all_libraries_from_config()
 
         details = (
             "Successfully reloaded all libraries. All object state was cleared and previous libraries were unloaded."
