@@ -13,6 +13,9 @@ from rich.console import Console
 
 from griptape_nodes.retained_mode.events.base_events import ResultPayload
 from griptape_nodes.retained_mode.events.os_events import (
+    CreateFileRequest,
+    CreateFileResultFailure,
+    CreateFileResultSuccess,
     FileSystemEntry,
     ListDirectoryRequest,
     ListDirectoryResultFailure,
@@ -23,6 +26,9 @@ from griptape_nodes.retained_mode.events.os_events import (
     ReadFileRequest,
     ReadFileResultFailure,
     ReadFileResultSuccess,
+    RenameFileRequest,
+    RenameFileResultFailure,
+    RenameFileResultSuccess,
 )
 from griptape_nodes.retained_mode.griptape_nodes import GriptapeNodes, logger
 from griptape_nodes.retained_mode.managers.event_manager import EventManager
@@ -59,6 +65,14 @@ class OSManager:
                 request_type=ReadFileRequest, callback=self.on_read_file_request
             )
 
+            event_manager.assign_manager_to_request_type(
+                request_type=CreateFileRequest, callback=self.on_create_file_request
+            )
+
+            event_manager.assign_manager_to_request_type(
+                request_type=RenameFileRequest, callback=self.on_rename_file_request
+            )
+
     def _get_workspace_path(self) -> Path:
         """Get the workspace path from config."""
         return GriptapeNodes.ConfigManager().workspace_path
@@ -204,17 +218,17 @@ class OSManager:
             sys.stdout.flush()  # Recommended here https://docs.python.org/3/library/os.html#os.execvpe
             os.execvp(args[0], args)  # noqa: S606
 
-    def on_open_associated_file_request(self, request: OpenAssociatedFileRequest) -> ResultPayload:  # noqa: C901, PLR0911, PLR0912
+    def on_open_associated_file_request(self, request: OpenAssociatedFileRequest) -> ResultPayload:  # noqa: PLR0911, PLR0912, PLR0915, C901
         # Validate that exactly one of path_to_file or file_entry is provided
         if request.path_to_file is None and request.file_entry is None:
             msg = "Either path_to_file or file_entry must be provided"
             logger.error(msg)
-            return OpenAssociatedFileResultFailure()
+            return OpenAssociatedFileResultFailure(result_details=msg)
 
         if request.path_to_file is not None and request.file_entry is not None:
             msg = "Only one of path_to_file or file_entry should be provided, not both"
             logger.error(msg)
-            return OpenAssociatedFileResultFailure()
+            return OpenAssociatedFileResultFailure(result_details=msg)
 
         # Get the file path to open
         if request.file_entry is not None:
@@ -227,29 +241,29 @@ class OSManager:
             # This should never happen due to validation above, but type checker needs it
             msg = "No valid file path provided"
             logger.error(msg)
-            return OpenAssociatedFileResultFailure()
+            return OpenAssociatedFileResultFailure(result_details=msg)
 
         # At this point, file_path_str is guaranteed to be a string
         if file_path_str is None:
             msg = "No valid file path provided"
             logger.error(msg)
-            return OpenAssociatedFileResultFailure()
+            return OpenAssociatedFileResultFailure(result_details=msg)
 
-        # Sanitize and validate the file path
+        # Sanitize and validate the path (file or directory)
         try:
             # Resolve the path (no workspace fallback for open requests)
             path = self._resolve_file_path(file_path_str, workspace_only=False)
         except (ValueError, RuntimeError):
             details = f"Invalid file path: '{file_path_str}'"
             logger.info(details)
-            return OpenAssociatedFileResultFailure()
+            return OpenAssociatedFileResultFailure(result_details=details)
 
-        if not path.exists() or not path.is_file():
-            details = f"File does not exist: '{path}'"
+        if not path.exists():
+            details = f"Path does not exist: '{path}'"
             logger.info(details)
-            return OpenAssociatedFileResultFailure()
+            return OpenAssociatedFileResultFailure(result_details=details)
 
-        logger.info("Attempting to open: %s on platform: %s", path, sys.platform)
+        logger.info("Attempting to open path: %s on platform: %s", path, sys.platform)
 
         try:
             platform_name = sys.platform
@@ -257,7 +271,7 @@ class OSManager:
                 # Linter complains but this is the recommended way on Windows
                 # We can ignore this warning as we've validated the path
                 os.startfile(str(path))  # noqa: S606 # pyright: ignore[reportAttributeAccessIssue]
-                logger.info("Started file on Windows: %s", path)
+                logger.info("Opened path on Windows: %s", path)
             elif self.is_mac():
                 # On macOS, open should be in a standard location
                 subprocess.run(  # noqa: S603
@@ -266,7 +280,7 @@ class OSManager:
                     capture_output=True,
                     text=True,
                 )
-                logger.info("Started file on macOS: %s", path)
+                logger.info("Opened path on macOS: %s", path)
             elif self.is_linux():
                 # Use full path to xdg-open to satisfy linter
                 # Common locations for xdg-open:
@@ -274,8 +288,9 @@ class OSManager:
 
                 xdg_path = next((p for p in xdg_paths if Path(p).exists()), None)
                 if not xdg_path:
-                    logger.info("xdg-open not found in standard locations")
-                    return OpenAssociatedFileResultFailure()
+                    details = "xdg-open not found in standard locations"
+                    logger.info(details)
+                    return OpenAssociatedFileResultFailure(result_details=details)
 
                 subprocess.run(  # noqa: S603
                     [xdg_path, str(path)],
@@ -283,24 +298,38 @@ class OSManager:
                     capture_output=True,
                     text=True,
                 )
-                logger.info("Started file on Linux: %s", path)
+                logger.info("Opened path on Linux: %s", path)
             else:
                 details = f"Unsupported platform: '{platform_name}'"
                 logger.info(details)
-                return OpenAssociatedFileResultFailure()
+                return OpenAssociatedFileResultFailure(result_details=details)
 
             return OpenAssociatedFileResultSuccess()
         except subprocess.CalledProcessError as e:
-            logger.error(
-                "Process error when opening file: return code=%s, stdout=%s, stderr=%s",
-                e.returncode,
-                e.stdout,
-                e.stderr,
+            details = (
+                f"Process error when opening file: return code={e.returncode}, stdout={e.stdout}, stderr={e.stderr}"
             )
-            return OpenAssociatedFileResultFailure()
+            logger.error(details)
+            return OpenAssociatedFileResultFailure(result_details=details)
         except Exception as e:
-            logger.error("Exception occurred when trying to open file: %s", type(e).__name__)
-            return OpenAssociatedFileResultFailure()
+            details = f"Exception occurred when trying to open path: {e}"
+            logger.error(details)
+            return OpenAssociatedFileResultFailure(result_details=details)
+
+    def _detect_mime_type(self, file_path: Path) -> str | None:
+        """Detect MIME type for a file. Returns None for directories or if detection fails."""
+        if file_path.is_dir():
+            return None
+
+        try:
+            mime_type, _ = mimetypes.guess_type(str(file_path), strict=True)
+            if mime_type is None:
+                mime_type = "text/plain"
+            return mime_type  # noqa: TRY300
+        except Exception as e:
+            msg = f"MIME type detection failed for {file_path}: {e}"
+            logger.warning(msg)
+            return "text/plain"
 
     def on_list_directory_request(self, request: ListDirectoryRequest) -> ResultPayload:  # noqa: C901, PLR0911
         """Handle a request to list directory contents."""
@@ -320,18 +349,18 @@ class OSManager:
             if not directory.exists():
                 msg = f"Directory does not exist: {directory}"
                 logger.error(msg)
-                return ListDirectoryResultFailure()
+                return ListDirectoryResultFailure(result_details=msg)
             if not directory.is_dir():
                 msg = f"Directory is not a directory: {directory}"
                 logger.error(msg)
-                return ListDirectoryResultFailure()
+                return ListDirectoryResultFailure(result_details=msg)
 
             # Check workspace constraints
             is_workspace_path, relative_or_abs_path = self._validate_workspace_path(directory)
             if request.workspace_only and not is_workspace_path:
                 msg = f"Directory is outside workspace: {directory}"
                 logger.error(msg)
-                return ListDirectoryResultFailure()
+                return ListDirectoryResultFailure(result_details=msg)
 
             entries = []
             try:
@@ -345,6 +374,7 @@ class OSManager:
                         stat = entry.stat()
                         # Get path relative to workspace if within workspace
                         is_entry_in_workspace, entry_path = self._validate_workspace_path(entry)
+                        mime_type = self._detect_mime_type(entry)
                         entries.append(
                             FileSystemEntry(
                                 name=entry.name,
@@ -352,6 +382,7 @@ class OSManager:
                                 is_dir=entry.is_dir(),
                                 size=stat.st_size,
                                 modified_time=stat.st_mtime,
+                                mime_type=mime_type,
                             )
                         )
                     except (OSError, PermissionError) as e:
@@ -362,7 +393,7 @@ class OSManager:
             except (OSError, PermissionError) as e:
                 msg = f"Error listing directory {directory}: {e}"
                 logger.error(msg)
-                return ListDirectoryResultFailure()
+                return ListDirectoryResultFailure(result_details=msg)
 
             # Return appropriate path format based on mode
             if request.workspace_only:
@@ -378,7 +409,7 @@ class OSManager:
         except Exception as e:
             msg = f"Unexpected error in list_directory: {type(e).__name__}: {e}"
             logger.error(msg)
-            return ListDirectoryResultFailure()
+            return ListDirectoryResultFailure(result_details=msg)
 
     def on_read_file_request(self, request: ReadFileRequest) -> ResultPayload:
         """Handle a request to read file contents with automatic text/binary detection."""
@@ -405,7 +436,7 @@ class OSManager:
             file_info = f" for file: {file_path}" if file_path is not None else ""
             msg = f"Validation error in read_file{file_info}: {e}"
             logger.error(msg)
-            return ReadFileResultFailure()
+            return ReadFileResultFailure(result_details=msg)
         except Exception as e:
             # Try to include file path in error message if available
             path_info = ""
@@ -416,7 +447,7 @@ class OSManager:
 
             msg = f"Unexpected error in read_file{path_info}: {type(e).__name__}: {e}"
             logger.error(msg)
-            return ReadFileResultFailure()
+            return ReadFileResultFailure(result_details=msg)
 
     def _read_file_content(
         self, file_path: Path, request: ReadFileRequest
@@ -703,3 +734,97 @@ class OSManager:
             logger.error("Attempted to clean up old files from %s, but no files could be deleted.")
 
         return removed_count > 0
+
+    def on_create_file_request(self, request: CreateFileRequest) -> ResultPayload:
+        """Handle a request to create a file or directory."""
+        try:
+            # Get the full path using the new method
+            full_path_str = request.get_full_path()
+
+            # Determine if path is absolute (not constrained to workspace)
+            is_absolute = Path(full_path_str).is_absolute()
+
+            # If workspace_only is True and path is absolute, it's outside workspace
+            if request.workspace_only and is_absolute:
+                msg = f"Absolute path is outside workspace: {full_path_str}"
+                logger.error(msg)
+                return CreateFileResultFailure(result_details=msg)
+
+            # Resolve path - if absolute, use as-is; if relative, align to workspace
+            if is_absolute:
+                file_path = Path(full_path_str).resolve()
+            else:
+                file_path = (self._get_workspace_path() / full_path_str).resolve()
+
+            # Check if it already exists - warn but treat as success
+            if file_path.exists():
+                msg = f"Path already exists: {file_path}"
+                logger.warning(msg)
+                return CreateFileResultSuccess(created_path=str(file_path))
+
+            # Create parent directories if needed
+            file_path.parent.mkdir(parents=True, exist_ok=True)
+
+            if request.is_directory:
+                file_path.mkdir()
+                logger.info("Created directory: %s", file_path)
+            # Create file with optional content
+            elif request.content is not None:
+                with file_path.open("w", encoding=request.encoding) as f:
+                    f.write(request.content)
+                logger.info("Created file with content: %s", file_path)
+            else:
+                file_path.touch()
+                logger.info("Created empty file: %s", file_path)
+
+            return CreateFileResultSuccess(created_path=str(file_path))
+
+        except Exception as e:
+            path_info = request.get_full_path() if hasattr(request, "get_full_path") else str(request.path)
+            msg = f"Failed to create {'directory' if request.is_directory else 'file'} at {path_info}: {e}"
+            logger.error(msg)
+            return CreateFileResultFailure(result_details=msg)
+
+    def on_rename_file_request(self, request: RenameFileRequest) -> ResultPayload:
+        """Handle a request to rename a file or directory."""
+        try:
+            # Resolve and validate old path
+            old_path = self._resolve_file_path(request.old_path, workspace_only=request.workspace_only is True)
+
+            # Resolve and validate new path
+            new_path = self._resolve_file_path(request.new_path, workspace_only=request.workspace_only is True)
+
+            # Check if old path exists
+            if not old_path.exists():
+                msg = f"Source path does not exist: {old_path}"
+                logger.error(msg)
+                return RenameFileResultFailure(result_details=msg)
+
+            # Check if new path already exists
+            if new_path.exists():
+                msg = f"Destination path already exists: {new_path}"
+                logger.error(msg)
+                return RenameFileResultFailure(result_details=msg)
+
+            # Check workspace constraints for both paths
+            is_old_in_workspace, _ = self._validate_workspace_path(old_path)
+            is_new_in_workspace, _ = self._validate_workspace_path(new_path)
+
+            if request.workspace_only and (not is_old_in_workspace or not is_new_in_workspace):
+                msg = f"One or both paths are outside workspace: {old_path} -> {new_path}"
+                logger.error(msg)
+                return RenameFileResultFailure(result_details=msg)
+
+            # Create parent directories for new path if needed
+            new_path.parent.mkdir(parents=True, exist_ok=True)
+
+            # Perform the rename operation
+            old_path.rename(new_path)
+            logger.info("Renamed: %s -> %s", old_path, new_path)
+
+            return RenameFileResultSuccess(old_path=str(old_path), new_path=str(new_path))
+
+        except Exception as e:
+            msg = f"Failed to rename {request.old_path} to {request.new_path}: {e}"
+            logger.error(msg)
+            return RenameFileResultFailure(result_details=msg)
