@@ -33,6 +33,7 @@ class AppEvents(BaseModel):
             "SingleExecutionStepRequest",
             "SingleNodeStepRequest",
             "ContinueExecutionStepRequest",
+            "SetLockNodeStateRequest",
         ]
     )
 
@@ -74,6 +75,7 @@ class Settings(BaseModel):
             "Exa": {"EXA_API_KEY": "$EXA_API_KEY"},
             "Grok": {"GROK_API_KEY": "$GROK_API_KEY"},
             "Groq": {"GROQ_API_KEY": "$GROQ_API_KEY"},
+            "Nvidia": {"NVIDIA_API_KEY": "$NVIDIA_API_KEY"},
             "Google": {"GOOGLE_API_KEY": "$GOOGLE_API_KEY", "GOOGLE_API_SEARCH_ID": "$GOOGLE_API_SEARCH_ID"},
             "Huggingface": {"HUGGINGFACE_HUB_ACCESS_TOKEN": "$HUGGINGFACE_HUB_ACCESS_TOKEN"},
             "LeonardoAI": {"LEONARDO_API_KEY": "$LEONARDO_API_KEY"},
@@ -95,3 +97,7 @@ class Settings(BaseModel):
     minimum_disk_space_gb_workflows: float = Field(
         default=1.0, description="Minimum disk space in GB required for saving workflows"
     )
+    synced_workflows_directory: str = Field(
+        default="synced_workflows",
+        description="Path to the synced workflows directory, relative to the workspace directory.",
+    )
